@@ -192,6 +192,12 @@
 
 ;;;; fops for loading symbols
 
+(defstruct (undefined-package
+            (:copier nil))
+  error)
+
+(declaim (freeze-type undefined-package))
+
 (defun aux-fop-intern (smallp package)
   (declare (optimize speed))
   (let* ((size (if smallp
@@ -206,11 +212,17 @@
       (read-string-as-unsigned-byte-32 *fasl-input-stream* buffer size)
       #!-sb-unicode
       (read-string-as-bytes *fasl-input-stream* buffer size))
-    (push-fop-table (without-package-locks
-                      (intern* buffer
-                               size
-                               package
-                               :no-copy t)))))
+    (if (undefined-package-p package)
+        (error 'simple-package-error
+               :format-control "Error finding package for symbol ~s:~% ~a"
+               :format-arguments
+               (list (subseq buffer 0 size)
+                     (undefined-package-error package)))
+        (push-fop-table (without-package-locks
+                          (intern* buffer
+                                   size
+                                   package
+                                   :no-copy t))))))
 
 (macrolet ((def (name code smallp package-form)
              `(define-fop (,name ,code)
@@ -260,7 +272,10 @@
       (read-string-as-bytes *fasl-input-stream* package-name)
       #!+sb-unicode
       (read-string-as-unsigned-byte-32 *fasl-input-stream* package-name))
-    (push-fop-table (find-undeleted-package-or-lose package-name))))
+    (push-fop-table
+     (handler-case (find-undeleted-package-or-lose package-name)
+       (simple-package-error (c)
+         (make-undefined-package :error (princ-to-string c)))))))
 
 ;;;; fops for loading numbers
 
@@ -420,6 +435,7 @@
             (setf (aref array (sb!vm:saetp-typecode saetp))
                   (sb!vm:saetp-n-bits saetp)))
       array)
+    #!+sb-doc
     "255 means bad entry.")
 (declaim (type (simple-array (unsigned-byte 8) (#.(1+ sb!vm:widetag-mask)))
                **saetp-bits-per-length**))
@@ -535,12 +551,13 @@
 (define-fop (fop-small-code 59 :stackp nil)
   (load-code (read-byte-arg) (read-halfword-arg)))
 
-(define-fop (fop-fdefinition 60)
-  (fdefinition-object (pop-stack) t))
+(define-fop (fop-fdefinition 60) ; should probably be 'fop-fdefn'
+  (find-or-create-fdefn (pop-stack)))
 
 (define-fop (fop-known-fun 65)
   (%coerce-name-to-fun (pop-stack)))
 
+#!-(or x86 x86-64)
 (define-fop (fop-sanctify-for-execution 61)
   (let ((component (pop-stack)))
     (sb!vm:sanctify-for-execution component)
@@ -632,6 +649,16 @@ a bug.~@:>")
 (define-fop (fop-assembler-routine 145)
   (error "cannot load assembler code except at cold load"))
 
+(define-fop (fop-symbol-tls-fixup 146)
+  (let* ((symbol (pop-stack))
+         (kind (pop-stack))
+         (code-object (pop-stack)))
+    (sb!vm:fixup-code-object code-object
+                             (read-word-arg)
+                             (ensure-symbol-tls-index symbol)
+                             kind)
+    code-object))
+
 (define-fop (fop-foreign-fixup 147)
   (let* ((kind (pop-stack))
          (code-object (pop-stack))
