@@ -19,18 +19,18 @@
 (defun current-dynamic-space-start () sb!vm:dynamic-space-start)
 #!-gencgc
 (defun current-dynamic-space-start ()
-  (sb!alien:extern-alien "current_dynamic_space" sb!alien:unsigned-long))
+  (extern-alien "current_dynamic_space" unsigned-long))
 
 #!-sb-fluid
 (declaim (inline dynamic-usage))
 #!+gencgc
 (defun dynamic-usage ()
-  (sb!alien:extern-alien "bytes_allocated" os-vm-size-t))
+  (extern-alien "bytes_allocated" os-vm-size-t))
 #!-gencgc
 (defun dynamic-usage ()
-  (the (unsigned-byte 32)
-       (- (sb!sys:sap-int (sb!c::dynamic-space-free-pointer))
-          (current-dynamic-space-start))))
+  (truly-the word
+             (- (sap-int (sb!c::dynamic-space-free-pointer))
+                (current-dynamic-space-start))))
 
 (defun static-space-usage ()
   (- (ash sb!vm:*static-space-free-pointer* sb!vm:n-fixnum-tag-bits)
@@ -42,15 +42,15 @@
 
 (defun control-stack-usage ()
   #!-stack-grows-downward-not-upward
-  (- (sb!sys:sap-int (sb!c::control-stack-pointer-sap))
-     (sb!sys:sap-int (sb!di::descriptor-sap sb!vm:*control-stack-start*)))
+  (- (sap-int (sb!c::control-stack-pointer-sap))
+     (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-start*)))
   #!+stack-grows-downward-not-upward
-  (- (sb!sys:sap-int (sb!di::descriptor-sap sb!vm:*control-stack-end*))
-     (sb!sys:sap-int (sb!c::control-stack-pointer-sap))))
+  (- (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-end*))
+     (sap-int (sb!c::control-stack-pointer-sap))))
 
 (defun binding-stack-usage ()
-  (- (sb!sys:sap-int (sb!c::binding-stack-pointer-sap))
-     (sb!sys:sap-int (sb!di::descriptor-sap sb!vm:*binding-stack-start*))))
+  (- (sap-int (sb!c::binding-stack-pointer-sap))
+     (sap-int (sb!di::descriptor-sap sb!vm:*binding-stack-start*))))
 
 ;;;; ROOM
 
@@ -133,7 +133,8 @@ and submit it as a patch."
 
 ;;;; GC hooks
 
-(defvar *after-gc-hooks* nil
+(!defvar *after-gc-hooks* nil
+  #!+sb-doc
   "Called after each garbage collection, except for garbage collections
 triggered during thread exits. In a multithreaded environment these hooks may
 run in any thread.")
@@ -141,13 +142,13 @@ run in any thread.")
 
 ;;;; internal GC
 
-(sb!alien:define-alien-routine collect-garbage sb!alien:int
-  (#!+gencgc last-gen #!-gencgc ignore sb!alien:int))
+(define-alien-routine collect-garbage int
+  (#!+gencgc last-gen #!-gencgc ignore int))
 
 #!+sb-thread
 (progn
-  (sb!alien:define-alien-routine gc-stop-the-world sb!alien:void)
-  (sb!alien:define-alien-routine gc-start-the-world sb!alien:void))
+  (define-alien-routine gc-stop-the-world void)
+  (define-alien-routine gc-start-the-world void))
 #!-sb-thread
 (progn
   (defun gc-stop-the-world ())
@@ -155,16 +156,16 @@ run in any thread.")
 
 #!+gencgc
 (progn
-  (sb!alien:define-alien-variable ("gc_logfile" %gc-logfile) (* char))
+  (define-alien-variable ("gc_logfile" %gc-logfile) (* char))
   (defun (setf gc-logfile) (pathname)
     (let ((new (when pathname
-                 (sb!alien:make-alien-string
+                 (make-alien-string
                   (native-namestring (translate-logical-pathname pathname)
                                      :as-file t))))
           (old %gc-logfile))
       (setf %gc-logfile new)
       (when old
-        (sb!alien:free-alien old))
+        (free-alien old))
       pathname))
   (defun gc-logfile ()
     #!+sb-doc
@@ -177,8 +178,9 @@ statistics are appended to it."
         (native-pathname val))))
   (declaim (inline dynamic-space-size))
   (defun dynamic-space-size ()
+    #!+sb-doc
     "Size of the dynamic space in bytes."
-    (sb!alien:extern-alien "dynamic_space_size" os-vm-size-t)))
+    (extern-alien "dynamic_space_size" os-vm-size-t)))
 
 ;;;; SUB-GC
 
@@ -213,7 +215,7 @@ statistics are appended to it."
 ;;; comparisons. Unlikely, but the cost of using a cons instead is too
 ;;; small to measure. -- JES, 2007-09-30
 (declaim (type cons *gc-epoch*))
-(defvar *gc-epoch* (cons nil nil))
+(!defvar *gc-epoch* '(nil . nil))
 
 (defun sub-gc (&key (gen 0))
   (cond (*gc-inhibit*
@@ -344,7 +346,7 @@ is true, all generations are collected. If GEN is provided, it can be
 used to specify the oldest generation guaranteed to be collected.
 
 On CheneyGC platforms arguments FULL and GEN take no effect: a full
-collection is always preformed."
+collection is always performed."
   #!+(and sb-doc (not gencgc))
   "Initiate a garbage collection.
 
@@ -361,7 +363,7 @@ guaranteed to be collected."
     (when (eq t (sub-gc :gen gen))
       (post-gc))))
 
-(define-alien-routine scrub-control-stack sb!alien:void)
+(define-alien-routine scrub-control-stack void)
 
 (defun unsafe-clear-roots (gen)
   #!-gencgc (declare (ignore gen))
@@ -393,11 +395,11 @@ On GENCGC platforms this is the nursery size, and defaults to 5% of dynamic
 space size.
 
 Note: currently changes to this value are lost when saving core."
-  (sb!alien:extern-alien "bytes_consed_between_gcs" os-vm-size-t))
+  (extern-alien "bytes_consed_between_gcs" os-vm-size-t))
 
 (defun (setf bytes-consed-between-gcs) (val)
   (declare (type index val))
-  (setf (sb!alien:extern-alien "bytes_consed_between_gcs" os-vm-size-t)
+  (setf (extern-alien "bytes_consed_between_gcs" os-vm-size-t)
         val))
 
 (declaim (inline maybe-handle-pending-gc))
@@ -498,6 +500,7 @@ promotion. Available on GENCGC platforms only.
 
 Experimental: interface subject to change."))
   (defun generation-average-age (generation)
+    #!+sb-doc
     "Average age of memory allocated to GENERATION: average number of times
 objects allocated to the generation have seen younger objects promoted to it.
 Available on GENCGC platforms only.
