@@ -417,9 +417,12 @@ code to be loaded.
 ;;; *LOOP-VARS*.
 (defvar *loop-declarations*)
 
+;;; Declarations for destructuring bindings
+(defvar *loop-desetq-declarations*)
+
 ;;; This is used by LOOP for destructuring binding, if it is doing
 ;;; that itself. See LOOP-MAKE-VAR.
-(defvar *loop-desetq-crocks*)
+(defvar *loop-desetq*)
 
 ;;; list of wrapping forms, innermost first, which go immediately
 ;;; inside the current set of parallel bindings being accumulated in
@@ -497,217 +500,32 @@ code to be loaded.
         (setq constantp nil value nil)))
     (values form constantp value)))
 
-;;;; LOOP iteration optimization
-
-(defvar *loop-duplicate-code* nil)
-
-(defvar *loop-iteration-flag-var* (make-symbol "LOOP-NOT-FIRST-TIME"))
-
-(defun loop-code-duplication-threshold (env)
-  (declare (ignore env))
-  (let (;; If we could read optimization declaration information (as
-        ;; with the DECLARATION-INFORMATION function (present in
-        ;; CLTL2, removed from ANSI standard) we could set these
-        ;; values flexibly. Without DECLARATION-INFORMATION, we have
-        ;; to set them to constants.
-        ;;
-        ;; except FIXME: we've lost all pretence of portability,
-        ;; considering this instead an internal implementation, so
-        ;; we're free to couple to our own representation of the
-        ;; environment.
-        (speed 1)
-        (space 1))
-    (+ 40 (* (- speed space) 10))))
-
-(sb!int:defmacro-mundanely loop-body (&environment env
-                                         prologue
-                                         before-loop
-                                         main-body
-                                         after-loop
-                                         epilogue
-                                         &aux rbefore rafter flagvar)
+(sb!int:defmacro-mundanely loop-body (prologue
+                                      before-loop
+                                      main-body
+                                      after-loop
+                                      epilogue)
   (unless (= (length before-loop) (length after-loop))
     (error "LOOP-BODY called with non-synched before- and after-loop lists"))
-  ;;All our work is done from these copies, working backwards from the end:
-  (setq rbefore (reverse before-loop) rafter (reverse after-loop))
-  (labels ((psimp (l)
-             (let ((ans nil))
-               (dolist (x l)
-                 (when x
-                   (push x ans)
-                   (when (and (consp x)
-                              (member (car x) '(go return return-from)))
-                     (return nil))))
-               (nreverse ans)))
-           (pify (l) (if (null (cdr l)) (car l) `(progn ,@l)))
-           (makebody ()
-             (let ((form `(tagbody
-                            ,@(psimp (append prologue (nreverse rbefore)))
-                         next-loop
-                            ,@(psimp (append main-body
-                                             (nreconc rafter
-                                                      `((go next-loop)))))
-                         end-loop
-                            ,@(psimp epilogue))))
-               (if flagvar `(let ((,flagvar nil)) ,form) form))))
-    (when (or *loop-duplicate-code* (not rbefore))
-      (return-from loop-body (makebody)))
-    ;; This outer loop iterates once for each not-first-time flag test
-    ;; generated plus once more for the forms that don't need a flag test.
-    (do ((threshold (loop-code-duplication-threshold env))) (nil)
-      (declare (fixnum threshold))
-      ;; Go backwards from the ends of before-loop and after-loop
-      ;; merging all the equivalent forms into the body.
-      (do () ((or (null rbefore) (not (equal (car rbefore) (car rafter)))))
-        (push (pop rbefore) main-body)
-        (pop rafter))
-      (unless rbefore (return (makebody)))
-      ;; The first forms in RBEFORE & RAFTER (which are the
-      ;; chronologically last forms in the list) differ, therefore
-      ;; they cannot be moved into the main body. If everything that
-      ;; chronologically precedes them either differs or is equal but
-      ;; is okay to duplicate, we can just put all of rbefore in the
-      ;; prologue and all of rafter after the body. Otherwise, there
-      ;; is something that is not okay to duplicate, so it and
-      ;; everything chronologically after it in rbefore and rafter
-      ;; must go into the body, with a flag test to distinguish the
-      ;; first time around the loop from later times. What
-      ;; chronologically precedes the non-duplicatable form will be
-      ;; handled the next time around the outer loop.
-      (do ((bb rbefore (cdr bb))
-           (aa rafter (cdr aa))
-           (lastdiff nil)
-           (count 0)
-           (inc nil))
-          ((null bb) (return-from loop-body (makebody)))        ; Did it.
-        (cond ((not (equal (car bb) (car aa))) (setq lastdiff bb count 0))
-              ((or (not (setq inc (estimate-code-size (car bb) env)))
-                   (> (incf count inc) threshold))
-               ;; Ok, we have found a non-duplicatable piece of code.
-               ;; Everything chronologically after it must be in the
-               ;; central body. Everything chronologically at and
-               ;; after LASTDIFF goes into the central body under a
-               ;; flag test.
-               (let ((then nil) (else nil))
-                 (do () (nil)
-                   (push (pop rbefore) else)
-                   (push (pop rafter) then)
-                   (when (eq rbefore (cdr lastdiff)) (return)))
-                 (unless flagvar
-                   (push `(setq ,(setq flagvar *loop-iteration-flag-var*)
-                                t)
-                         else))
-                 (push `(if ,flagvar ,(pify (psimp then)) ,(pify (psimp else)))
-                       main-body))
-               ;; Everything chronologically before lastdiff until the
-               ;; non-duplicatable form (CAR BB) is the same in
-               ;; RBEFORE and RAFTER, so just copy it into the body.
-               (do () (nil)
-                 (pop rafter)
-                 (push (pop rbefore) main-body)
-                 (when (eq rbefore (cdr bb)) (return)))
-               (return)))))))
-
-(defun duplicatable-code-p (expr env)
-  (if (null expr) 0
-      (let ((ans (estimate-code-size expr env)))
-        (declare (fixnum ans))
-        ;; @@@@ Use (DECLARATION-INFORMATION 'OPTIMIZE ENV) here to
-        ;; get an alist of optimize quantities back to help quantify
-        ;; how much code we are willing to duplicate.
-        ans)))
-
-(defvar *special-code-sizes*
-        '((return 0) (progn 0)
-          (null 1) (not 1) (eq 1) (car 1) (cdr 1)
-          (when 1) (unless 1) (if 1)
-          (caar 2) (cadr 2) (cdar 2) (cddr 2)
-          (caaar 3) (caadr 3) (cadar 3) (caddr 3)
-          (cdaar 3) (cdadr 3) (cddar 3) (cdddr 3)
-          (caaaar 4) (caaadr 4) (caadar 4) (caaddr 4)
-          (cadaar 4) (cadadr 4) (caddar 4) (cadddr 4)
-          (cdaaar 4) (cdaadr 4) (cdadar 4) (cdaddr 4)
-          (cddaar 4) (cddadr 4) (cdddar 4) (cddddr 4)))
-
-(defvar *estimate-code-size-punt*
-        '(block
-           do do* dolist
-           flet
-           labels lambda let let* locally
-           macrolet multiple-value-bind
-           prog prog*
-           symbol-macrolet
-           tagbody
-           unwind-protect
-           with-open-file))
-
-(defun destructuring-size (x)
-  (do ((x x (cdr x)) (n 0 (+ (destructuring-size (car x)) n)))
-      ((atom x) (+ n (if (null x) 0 1)))))
-
-(defun estimate-code-size (x env)
-  (catch 'estimate-code-size
-    (estimate-code-size-1 x env)))
-
-(defun estimate-code-size-1 (x env)
-  (flet ((list-size (l)
-           (let ((n 0))
-             (declare (fixnum n))
-             (dolist (x l n) (incf n (estimate-code-size-1 x env))))))
-    ;;@@@@ ???? (declare (function list-size (list) fixnum))
-    (cond ((constantp x) 1)
-          ((symbolp x) (multiple-value-bind (new-form expanded-p)
-                           (sb!int:%macroexpand-1 x env)
-                         (if expanded-p
-                             (estimate-code-size-1 new-form env)
-                             1)))
-          ((atom x) 1) ;; ??? self-evaluating???
-          ((symbolp (car x))
-           (let ((fn (car x)) (tem nil) (n 0))
-             (declare (symbol fn) (fixnum n))
-             (macrolet ((f (overhead &optional (args nil args-p))
-                          `(the fixnum (+ (the fixnum ,overhead)
-                                          (the fixnum
-                                               (list-size ,(if args-p
-                                                               args
-                                                             '(cdr x))))))))
-               (cond ((setq tem (get fn 'estimate-code-size))
-                      (typecase tem
-                        (fixnum (f tem))
-                        (t (funcall tem x env))))
-                     ((setq tem (assoc fn *special-code-sizes*))
-                      (f (second tem)))
-                     ((eq fn 'cond)
-                      (dolist (clause (cdr x) n)
-                        (incf n (list-size clause)) (incf n)))
-                     ((eq fn 'desetq)
-                      (do ((l (cdr x) (cdr l))) ((null l) n)
-                        (setq n (+ n
-                                   (destructuring-size (car l))
-                                   (estimate-code-size-1 (cadr l) env)))))
-                     ((member fn '(setq psetq))
-                      (do ((l (cdr x) (cdr l))) ((null l) n)
-                        (setq n (+ n (estimate-code-size-1 (cadr l) env) 1))))
-                     ((eq fn 'go) 1)
-                     ((eq fn 'function)
-                      (if (sb!int:legal-fun-name-p (cadr x))
-                          1
-                          ;; FIXME: This tag appears not to be present
-                          ;; anywhere.
-                          (throw 'duplicatable-code-p nil)))
-                     ((eq fn 'multiple-value-setq)
-                      (f (length (second x)) (cddr x)))
-                     ((eq fn 'return-from)
-                      (1+ (estimate-code-size-1 (third x) env)))
-                     ((or (special-operator-p fn)
-                          (member fn *estimate-code-size-punt*))
-                      (throw 'estimate-code-size nil))
-                     (t (multiple-value-bind (new-form expanded-p)
-                            (sb!int:%macroexpand-1 x env)
-                          (if expanded-p
-                              (estimate-code-size-1 new-form env)
-                              (f 3))))))))
-          (t (throw 'estimate-code-size nil)))))
+  ;; All our work is done from these copies, working backwards from the end
+  (let ((rbefore (reverse before-loop))
+        (rafter (reverse after-loop)))
+    ;; Go backwards from the ends of before-loop and after-loop
+    ;; merging all the equivalent forms into the body.
+    (do ()
+        ((or (null rbefore)
+             (not (equal (car rbefore) (car rafter)))))
+      (push (pop rbefore) main-body)
+      (pop rafter))
+    `(tagbody
+        ,@(remove nil prologue)
+        ,@(nreverse (remove nil rbefore))
+      next-loop
+        ,@(remove nil main-body)
+        ,@(nreverse (remove nil rafter))
+        (go next-loop)
+      end-loop
+        ,@(remove nil epilogue))))
 
 ;;;; loop errors
 
@@ -752,7 +570,7 @@ code to be loaded.
   (let ((*ignores* nil))
     (declare (special *ignores*))
     (let ((d-var-lambda-list (subst-gensyms-for-nil lambda-list)))
-      `(destructuring-bind ,d-var-lambda-list
+      `(destructuring-bind (&optional ,@d-var-lambda-list)
            ,arg-list
          (declare (ignore ,@*ignores*))
          ,@body))))
@@ -771,7 +589,8 @@ code to be loaded.
         (*loop-vars* nil)
         (*loop-named-vars* nil)
         (*loop-declarations* nil)
-        (*loop-desetq-crocks* nil)
+        (*loop-desetq-declarations* nil)
+        (*loop-desetq* nil)
         (*loop-bind-stack* nil)
         (*loop-prologue* nil)
         (*loop-wrappers* nil)
@@ -797,19 +616,17 @@ code to be loaded.
                      ,(nreconc *loop-epilogue*
                                (nreverse *loop-after-epilogue*)))))
       (dolist (entry *loop-bind-stack*)
-        (let ((vars (first entry))
-              (dcls (second entry))
-              (crocks (third entry))
-              (wrappers (fourth entry)))
+        (destructuring-bind (vars dcls desetq desetq-decls wrappers) entry
           (dolist (w wrappers)
             (setq answer (append w (list answer))))
-          (when (or vars dcls crocks)
+          (when (or vars dcls desetq)
             (let ((forms (list answer)))
-              ;;(when crocks (push crocks forms))
-              (when dcls (push `(declare ,@dcls) forms))
+              (when desetq-decls
+                (push `(declare ,@desetq-decls) forms))
               (setq answer `(,(if vars 'let 'locally)
                              ,vars
-                             ,@(loop-build-destructuring-bindings crocks
+                             (declare ,@dcls)
+                             ,@(loop-build-destructuring-bindings desetq
                                                                   forms)))))))
       (do () (nil)
         (setq answer `(block ,(pop *loop-names*) ,answer))
@@ -900,33 +717,42 @@ code to be loaded.
 ;;;; loop types
 
 (defun loop-typed-init (data-type &optional step-var-p)
-  (cond ((null data-type)
-         nil)
-        ((sb!xc:subtypep data-type 'number)
-         (let ((init (if step-var-p 1 0)))
-           (flet ((like (&rest types)
-                    (coerce init (find-if (lambda (type)
-                                            (sb!xc:subtypep data-type type))
-                                          types))))
-             (cond ((sb!xc:subtypep data-type 'float)
-                    (like 'single-float 'double-float
-                          'short-float 'long-float 'float))
-                   ((sb!xc:subtypep data-type '(complex float))
-                    (like '(complex single-float)
-                          '(complex double-float)
-                          '(complex short-float)
-                          '(complex long-float)
-                          '(complex float)))
-                   (t
-                    init)))))
-        ((sb!xc:subtypep data-type 'vector)
-         (let ((ctype (sb!kernel:specifier-type data-type)))
-           (when (sb!kernel:array-type-p ctype)
-             (let ((etype (sb!kernel:type-*-to-t
-                           (sb!kernel:array-type-specialized-element-type ctype))))
-               (make-array 0 :element-type (sb!kernel:type-specifier etype))))))
-        (t
-         nil)))
+  ;; FIXME: can't tell if unsupplied or NIL, but it has to be rare.
+  (when data-type
+   (let ((ctype (sb!kernel:specifier-type data-type)))
+     ;; FIXME: use the ctype for the rest of the type operations, now
+     ;; that it's parsed.
+     (cond ((eql ctype sb!kernel:*empty-type*)
+            (values nil t))
+           ((sb!xc:subtypep data-type 'number)
+            (let ((init (if step-var-p 1 0)))
+              (flet ((like (&rest types)
+                       (coerce init (find-if (lambda (type)
+                                               (sb!xc:subtypep data-type type))
+                                             types))))
+                (cond ((sb!xc:subtypep data-type 'float)
+                       (like 'single-float 'double-float
+                             'short-float 'long-float 'float))
+                      ((sb!xc:subtypep data-type '(complex float))
+                       (like '(complex single-float)
+                             '(complex double-float)
+                             '(complex short-float)
+                             '(complex long-float)
+                             '(complex float)))
+                      (t
+                       init)))))
+           ((sb!xc:subtypep data-type 'vector)
+            (when (sb!kernel:array-type-p ctype)
+              (let ((etype (sb!kernel:type-*-to-t
+                            (sb!kernel:array-type-specialized-element-type ctype))))
+                (make-array 0 :element-type (sb!kernel:type-specifier etype)))))
+           #!+sb-unicode
+           ((sb!xc:subtypep data-type 'extended-char)
+            (code-char sb!int:base-char-code-limit))
+           ((sb!xc:subtypep data-type 'character)
+            #\x)
+           (t
+            nil)))))
 
 (defun loop-optional-type (&optional variable)
   ;; No variable specified implies that no destructuring is permissible.
@@ -999,74 +825,136 @@ code to be loaded.
 ;;;; loop variables
 
 (defun loop-bind-block ()
-  (when (or *loop-vars* *loop-declarations* *loop-wrappers*)
+  (when (or *loop-vars* *loop-declarations* *loop-wrappers*
+            *loop-desetq*)
     (push (list (nreverse *loop-vars*)
                 *loop-declarations*
-                *loop-desetq-crocks*
+                *loop-desetq*
+                *loop-desetq-declarations*
                 *loop-wrappers*)
           *loop-bind-stack*)
     (setq *loop-vars* nil
           *loop-declarations* nil
-          *loop-desetq-crocks* nil
+          *loop-desetq* nil
+          *loop-desetq-declarations* nil
           *loop-wrappers* nil)))
 
-(defun loop-var-p (name)
-  (do ((entry *loop-bind-stack* (cdr entry)))
-      (nil)
-    (cond
-      ((null entry) (return nil))
-      ((assoc name (caar entry) :test #'eq) (return t)))))
+(defun check-var-name (name &optional (context ""))
+  (labels ((map-name (function name)
+             (do ((x (pop name) (pop name)))
+                 (())
+               (typecase x
+                 (null)
+                 (cons (map-name function x))
+                 (symbol (funcall function x))
+                 (t
+                  (loop-error "Bad variable ~s~a" x context)))
+               (typecase name
+                 (cons)
+                 (null
+                  (return))
+                 (symbol
+                  (funcall function name)
+                  (return))
+                 (t
+                  (loop-error "Bad variable ~s~a" name context)))))
+           (duplicate (x)
+             (loop-error "Duplicated variable ~s~a" x context))
+           (find-in-desetq (name desetqs)
+             (do* ((desetq desetqs (cddr desetq))
+                   (var (car desetq) (car desetq)))
+                  ((null desetq))
+               (map-name (lambda (x)
+                           (when (eql name x)
+                             (duplicate name)))
+                         var))))
+    (cond ((consp name)
+           (map-name (lambda (x) (check-var-name x context)) name))
+          ((assoc name *loop-vars*)
+           (duplicate name))
+          ((find-in-desetq name *loop-desetq*))
+          (t
+           (do ((entry *loop-bind-stack* (cdr entry)))
+               (nil)
+             (cond
+               ((null entry) (return nil))
+               ((assoc name (caar entry) :test #'eq)
+                (duplicate name))
+               (t
+                (find-in-desetq name (caddar entry)))))))))
 
 (defun loop-make-var (name initialization dtype &optional step-var-p)
   (cond ((null name)
          (setq name (gensym "LOOP-IGNORE-"))
-         (push (list name initialization) *loop-vars*)
+         (push (list name (or initialization (loop-typed-init dtype step-var-p)))
+               *loop-vars*)
          (push `(ignore ,name) *loop-declarations*)
          (loop-declare-var name dtype))
         ((atom name)
-         (when (or (assoc name *loop-vars*)
-                   (loop-var-p name))
-           (loop-error "duplicated variable ~S in a LOOP binding" name))
-         (unless (symbolp name)
-           (loop-error "bad variable ~S somewhere in LOOP" name))
-         (loop-declare-var name dtype step-var-p initialization)
+         (check-var-name name)
+         (loop-declare-var name dtype :step-var-p step-var-p
+                                      :initialization initialization)
          ;; We use ASSOC on this list to check for duplications (above),
          ;; so don't optimize out this list:
          (push (list name (or initialization (loop-typed-init dtype step-var-p)))
                *loop-vars*))
         (initialization
+         (check-var-name name)
          (let ((newvar (gensym "LOOP-DESTRUCTURE-")))
-           (loop-declare-var name dtype)
+           (loop-declare-var name dtype :desetq t)
            (push (list newvar initialization) *loop-vars*)
-           ;; *LOOP-DESETQ-CROCKS* gathered in reverse order.
-           (setq *loop-desetq-crocks*
-                 (list* name newvar *loop-desetq-crocks*))))
-        (t (let ((tcar nil) (tcdr nil))
-             (if (atom dtype) (setq tcar (setq tcdr dtype))
-                 (setq tcar (car dtype) tcdr (cdr dtype)))
-             (loop-make-var (car name) nil tcar)
-             (loop-make-var (cdr name) nil tcdr))))
+           ;; *LOOP-DESETQ* gathered in reverse order.
+           (setq *loop-desetq*
+                 (list* name newvar *loop-desetq*))))
+        (t
+         (let ((tcar nil) (tcdr nil))
+           (if (atom dtype) (setq tcar (setq tcdr dtype))
+               (setq tcar (car dtype) tcdr (cdr dtype)))
+           (loop-make-var (car name) nil tcar)
+           (when (cdr name)
+             (loop-make-var (cdr name) nil tcdr)))))
   name)
 
-(defun loop-declare-var (name dtype &optional step-var-p initialization)
+;;; Find a suitable type for  default initialization
+(defun type-for-default-init (type &optional step-var-p)
+  (multiple-value-bind (init empty-type)
+      (loop-typed-init type step-var-p)
+    (values
+     (cond (empty-type
+            ;; Don't wrap empty types `(or ...), otherwise the will no
+            ;; longer be empty and the compiler won't produce
+            ;; warnings.
+            type)
+           ((sb!xc:typep init type)
+            type)
+           (t
+            `(or ,(type-of init) ,type)))
+     init)))
+
+(defun loop-declare-var (name dtype &key step-var-p initialization
+                                         desetq)
   (cond ((or (null name) (null dtype) (eq dtype t)) nil)
         ((symbolp name)
          (unless (or (sb!xc:subtypep t dtype)
                      (and (eq (find-package :cl) (symbol-package name))
                           (eq :special (sb!int:info :variable :kind name))))
-           (let ((dtype (if initialization
-                            dtype
-                            (let ((init (loop-typed-init dtype step-var-p)))
-                              (if (sb!xc:typep init dtype)
-                                  dtype
-                                  `(or ,(type-of init) ,dtype))))))
-             (push `(type ,dtype ,name) *loop-declarations*))))
+           (let ((dtype `(type ,(if initialization
+                                    dtype
+                                    (type-for-default-init dtype step-var-p))
+                               ,name)))
+             (if desetq
+                 (push dtype *loop-desetq-declarations*)
+                 (push dtype *loop-declarations*)))))
         ((consp name)
          (cond ((consp dtype)
-                (loop-declare-var (car name) (car dtype))
-                (loop-declare-var (cdr name) (cdr dtype)))
-               (t (loop-declare-var (car name) dtype)
-                  (loop-declare-var (cdr name) dtype))))
+                (loop-declare-var (car name) (car dtype)
+                                  :desetq desetq)
+                (loop-declare-var (cdr name) (cdr dtype)
+                                  :desetq desetq))
+               (t (loop-declare-var (car name) dtype
+                                    :desetq desetq)
+                  (loop-declare-var (cdr name) dtype
+                                    :desetq desetq))))
         (t (error "invalid LOOP variable passed in: ~S" name))))
 
 (defun loop-maybe-bind-form (form data-type)
@@ -1156,9 +1044,19 @@ code to be loaded.
   class
   (history nil)
   (tempvars nil)
+  specified-type
   dtype
   (data nil)) ;collector-specific data
 
+(sb!int:defmacro-mundanely with-sum-count (lc &body body)
+  (let* ((type (loop-collector-dtype lc))
+         (temp-var (car (loop-collector-tempvars lc))))
+    (multiple-value-bind (type init)
+        (type-for-default-init type)
+      `(let ((,temp-var ,init))
+         (declare (type ,type ,temp-var))
+         ,@body))))
+
 (defun loop-get-collection-info (collector class default-type)
   (let ((form (loop-get-form))
         (name (when (loop-tequal (car *loop-source-code*) 'into)
@@ -1168,28 +1066,41 @@ code to be loaded.
       (loop-error "The value accumulation recipient name, ~S, is not a symbol." name))
     (unless name
       (loop-disallow-aggregate-booleans))
-    (let ((dtype (or (loop-optional-type) default-type))
-          (cruft (find (the symbol name) *loop-collection-cruft*
-                       :key #'loop-collector-name)))
+    (let* ((specified-type (loop-optional-type))
+           (dtype (or specified-type default-type))
+           (cruft (find (the symbol name) *loop-collection-cruft*
+                        :key #'loop-collector-name)))
       (cond ((not cruft)
-             (when (and name (loop-var-p name))
-               (loop-error "Variable ~S in INTO clause is a duplicate" name))
+             (check-var-name name " in INTO clause")
              (push (setq cruft (make-loop-collector
-                                 :name name :class class
-                                 :history (list collector) :dtype dtype))
+                                :name name :class class
+                                :history (list collector)
+                                :specified-type specified-type
+                                :dtype dtype))
                    *loop-collection-cruft*))
             (t (unless (eq (loop-collector-class cruft) class)
                  (loop-error
-                   "incompatible kinds of LOOP value accumulation specified for collecting~@
+                  "incompatible kinds of LOOP value accumulation specified for collecting~@
                     ~:[as the value of the LOOP~;~:*INTO ~S~]: ~S and ~S"
-                   name (car (loop-collector-history cruft)) collector))
-               (unless (equal dtype (loop-collector-dtype cruft))
-                 (loop-warn
-                   "unequal datatypes specified in different LOOP value accumulations~@
-                   into ~S: ~S and ~S"
-                   name dtype (loop-collector-dtype cruft))
-                 (when (eq (loop-collector-dtype cruft) t)
-                   (setf (loop-collector-dtype cruft) dtype)))
+                  name (car (loop-collector-history cruft)) collector))
+               (cond ((equal dtype (loop-collector-dtype cruft)))
+                     ((and (null specified-type)
+                           (null (loop-collector-specified-type cruft)))
+                      ;; Unionize types only for default types, most
+                      ;; likely, SUM and COUNT which have number and
+                      ;; fixnum respectively.
+                      (setf (loop-collector-dtype cruft)
+                            (sb!kernel:type-specifier
+                             (sb!kernel:type-union
+                              (sb!kernel:specifier-type dtype)
+                              (sb!kernel:specifier-type (loop-collector-dtype cruft))))))
+                     (t
+                      (loop-warn
+                       "unequal datatypes specified in different LOOP value accumulations~@
+                        into ~S: ~S and ~S"
+                       name dtype (loop-collector-dtype cruft))
+                      (when (eq (loop-collector-dtype cruft) t)
+                        (setf (loop-collector-dtype cruft) dtype))))
                (push collector (loop-collector-history cruft))))
       (values cruft form))))
 
@@ -1223,12 +1134,11 @@ code to be loaded.
     (let ((tempvars (loop-collector-tempvars lc)))
       (unless tempvars
         (setf (loop-collector-tempvars lc)
-              (setq tempvars (list (loop-make-var
-                                     (or (loop-collector-name lc)
-                                         (gensym "LOOP-SUM-"))
-                                     nil (loop-collector-dtype lc)))))
+              (setq tempvars (list (or (loop-collector-name lc)
+                                  (gensym "LOOP-SUM-")))))
         (unless (loop-collector-name lc)
-          (loop-emit-final-value (car (loop-collector-tempvars lc)))))
+          (loop-emit-final-value (car (loop-collector-tempvars lc))))
+        (push `(with-sum-count ,lc) *loop-wrappers*))
       (loop-emit-body
         (if (eq specifically 'count)
             `(when ,form
@@ -1246,13 +1156,13 @@ code to be loaded.
       (unless data
         (setf (loop-collector-data lc)
               (setq data (make-loop-minimax
-                           (or (loop-collector-name lc)
-                               (gensym "LOOP-MAXMIN-"))
-                           (loop-collector-dtype lc))))
+                          (or (loop-collector-name lc)
+                              (gensym "LOOP-MAXMIN-"))
+                          (loop-collector-dtype lc))))
         (unless (loop-collector-name lc)
-          (loop-emit-final-value (loop-minimax-answer-variable data))))
+          (loop-emit-final-value (loop-minimax-answer-variable data)))
+        (push `(with-minimax-value ,data) *loop-wrappers*))
       (loop-note-minimax-operation specifically data)
-      (push `(with-minimax-value ,data) *loop-wrappers*)
       (loop-emit-body `(loop-accumulate-minimax-value ,data
                                                       ,specifically
                                                       ,form)))))
@@ -1286,8 +1196,13 @@ code to be loaded.
 
 (defun loop-do-repeat ()
   (loop-disallow-conditional :repeat)
-  (let ((form (loop-get-form))
-        (type 'integer))
+  (let* ((form (loop-get-form))
+         (type (cond ((not (realp form))
+                      'integer)
+                     ((plusp form)
+                      `(mod ,(1+ (ceiling form))))
+                     (t
+                      `(integer ,(ceiling form))))))
     (let ((var (loop-make-var (gensym "LOOP-REPEAT-") `(ceiling ,form) type)))
       (push `(if (<= ,var 0) (go end-loop) (decf ,var)) *loop-before-loop*)
       (push `(if (<= ,var 0) (go end-loop) (decf ,var)) *loop-after-body*)
@@ -1310,8 +1225,6 @@ code to be loaded.
                      (loop-pop-source)
                      (loop-get-form))
                     (t nil)))
-    (when (and var (loop-var-p var))
-      (loop-error "Variable ~S has already been used" var))
     (loop-make-var var val dtype)
     (if (loop-tequal (car *loop-source-code*) :and)
         (loop-pop-source)
@@ -1677,8 +1590,7 @@ code to be loaded.
          (start-constantp nil)
          (limit-given nil) ; T when prep phrase has specified end
          (limit-constantp nil)
-         (limit-value nil)
-         )
+         (limit-value nil))
      (flet ((assert-index-for-arithmetic (index)
               (unless (atom index)
                 (loop-error "Arithmetic index must be an atom."))))
@@ -1736,34 +1648,33 @@ code to be loaded.
        (when (and sequence-variable (not sequencep))
          (loop-error "missing OF or IN phrase in sequence path"))
        ;; Now fill in the defaults.
-       (if start-given
-           (when limit-given
-             ;; if both start and limit are given, they had better both
-             ;; be REAL.  We already enforce the REALness of LIMIT,
-             ;; above; here's the KLUDGE to enforce the type of START.
-             (flet ((type-declaration-of (x)
-                      (and (eq (car x) 'type) (caddr x))))
-               (let ((decl (find indexv *loop-declarations*
-                                 :key #'type-declaration-of))
-                     (%decl (find indexv *loop-declarations*
-                                  :key #'type-declaration-of
-                                  :from-end t)))
-                 (sb!int:aver (eq decl %decl))
-                 (when decl
-                   (setf (cadr decl)
-                         `(and real ,(cadr decl)))))))
-           ;; default start
-           ;; DUPLICATE KLUDGE: loop-make-var generates a temporary
-           ;; symbol for indexv if it is NIL. See also the comment in
-           ;; the (:from :downfrom :upfrom) case
-           (progn
-             (assert-index-for-arithmetic indexv)
-             (setq indexv
-                   (loop-make-var
-                      indexv
-                      (setq start-constantp t
-                            start-value (or (loop-typed-init indexv-type) 0))
-                      `(and ,indexv-type real)))))
+       (cond ((not start-given)
+              ;; default start
+              ;; DUPLICATE KLUDGE: loop-make-var generates a temporary
+              ;; symbol for indexv if it is NIL. See also the comment in
+              ;; the (:from :downfrom :upfrom) case
+              (assert-index-for-arithmetic indexv)
+              (setq indexv
+                    (loop-make-var
+                     indexv
+                     (setq start-constantp t
+                           start-value (or (loop-typed-init indexv-type) 0))
+                     `(and ,indexv-type real))))
+             (limit-given
+              ;; if both start and limit are given, they had better both
+              ;; be REAL.  We already enforce the REALness of LIMIT,
+              ;; above; here's the KLUDGE to enforce the type of START.
+              (flet ((type-declaration-of (x)
+                       (and (eq (car x) 'type) (caddr x))))
+                (let ((decl (find indexv *loop-declarations*
+                                  :key #'type-declaration-of))
+                      (%decl (find indexv *loop-declarations*
+                                   :key #'type-declaration-of
+                                   :from-end t)))
+                  (sb!int:aver (eq decl %decl))
+                  (when decl
+                    (setf (cadr decl)
+                          `(and real ,(cadr decl))))))))
        (cond ((member dir '(nil :up))
               (when (or limit-given default-top)
                 (unless limit-given
@@ -1933,10 +1844,13 @@ code to be loaded.
                          (nconc (loop-list-collection nconc))
                          (nconcing (loop-list-collection nconc))
                          (count (loop-sum-collection count
-                                                     real
-                                                     fixnum))
+                                 ;; This could be REAL, but when
+                                 ;; combined with SUM, it has to be
+                                 ;; NUMBER.
+                                 number
+                                 fixnum))
                          (counting (loop-sum-collection count
-                                                        real
+                                                        number
                                                         fixnum))
                          (sum (loop-sum-collection sum number number))
                          (summing (loop-sum-collection sum number number))
