@@ -362,7 +362,7 @@
         (let ((char (stream-read-char stream)))
           (if (eq char :eof)
               (eof-or-lose stream eof-error-p eof-value)
-              char)))))
+              (the character char))))))
 
 #!-sb-fluid (declaim (inline ansi-stream-unread-char))
 (defun ansi-stream-unread-char (character stream)
@@ -421,7 +421,7 @@
         (let ((char (stream-read-char-no-hang stream)))
           (if (eq char :eof)
               (eof-or-lose stream eof-error-p eof-value)
-              char)))))
+              (the (or character null) char))))))
 
 #!-sb-fluid (declaim (inline ansi-stream-clear-input))
 (defun ansi-stream-clear-input (stream)
@@ -449,10 +449,10 @@
   (if (ansi-stream-p stream)
       (ansi-stream-read-byte stream eof-error-p eof-value nil)
       ;; must be Gray streams FUNDAMENTAL-STREAM
-      (let ((char (stream-read-byte stream)))
-        (if (eq char :eof)
+      (let ((byte (stream-read-byte stream)))
+        (if (eq byte :eof)
             (eof-or-lose stream eof-error-p eof-value)
-            char))))
+            (the integer byte)))))
 
 ;;; Read NUMBYTES bytes into BUFFER beginning at START, and return the
 ;;; number of bytes read.
@@ -697,6 +697,12 @@
   integer)
 
 
+;;; Meta: the following comment is mostly true, but gray stream support
+;;;   is already incorporated into the definitions within this file.
+;;;   But these need to redefinable, otherwise the relative order of
+;;;   loading sb-simple-streams and any user-defined code which executes
+;;;   (F #'read-char ...) is sensitive to the order in which those
+;;;   are loaded, though insensitive at compile-time.
 ;;; (These were inline throughout this file, but that's not appropriate
 ;;; globally.  And we must not inline them in the rest of this file if
 ;;; dispatch to gray or simple streams is to work, since both redefine
@@ -1168,7 +1174,7 @@
              (:constructor internal-make-string-input-stream
                            (string current end))
              (:copier nil))
-  (string (missing-arg) :type simple-string)
+  (string (missing-arg) :type simple-string :read-only t)
   (current (missing-arg) :type index)
   (end (missing-arg) :type index))
 
@@ -1228,8 +1234,7 @@
                  (:start 0)
                  (:end (string-input-stream-end stream))
                  ;; We allow moving position beyond EOF. Errors happen
-                 ;; on read, not move -- or the user may extend the
-                 ;; input string.
+                 ;; on read, not move.
                  (t arg1)))
          (string-input-stream-current stream)))
     ;; According to ANSI: "Should signal an error of type type-error
@@ -1250,8 +1255,17 @@
   (declare (type string string)
            (type index start)
            (type (or index null) end))
+  ;; FIXME: very inefficient if the input string is, say a 100000-character
+  ;; adjustable string but (- END START) is 100 characters. We should use
+  ;; SUBSEQ instead of coercing the whole string. And if STRING is non-simple
+  ;; but has element type CHARACTER, wouldn't it work to just use the
+  ;; underlying simple-string since INTERNAL-MAKE- accepts bounding indices
+  ;; that can be fudged to deal with any offset?
+  ;; And (for unicode builds) if the input is BASE-STRING, we should use
+  ;; MAKE-ARRAY and REPLACE to coerce just the specified piece.
   (let* ((string (coerce string '(simple-array character (*)))))
-    ;; FIXME: Why WITH-ARRAY-DATA, since the array is already simple?
+    ;; Why WITH-ARRAY-DATA, since the array is already simple?
+    ;; because it's a nice abstract way to check the START and END.
     (with-array-data ((string string) (start start) (end end))
       (internal-make-string-input-stream
        string ;; now simple
@@ -1271,10 +1285,6 @@
 
 (defparameter *string-output-stream-buffer-initial-size* 64)
 
-#!-sb-fluid
-(declaim (inline string-output-string-stream-buffer
-                 string-output-string-stream-pointer
-                 string-output-string-stream-index))
 (defstruct (string-output-stream
             (:include ansi-stream
                       (out #'string-ouch)
@@ -1569,7 +1579,7 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
 
 ;;; FIXME: need to support (VECTOR NIL), ideally without destroying all hope
 ;;; of efficiency.
-(declaim (inline vector-with-fill-pointer))
+(declaim (inline vector-with-fill-pointer-p))
 (defun vector-with-fill-pointer-p (x)
   (and (vectorp x)
        (array-has-fill-pointer-p x)))
