@@ -182,12 +182,14 @@ This is SETFable."
                        ;; it could just save and restore the number-stack pointer once,
                        ;; instead of doing multiple decrements if there are multiple bindings.
                        #!-(or x86 x86-64)
-                       `((let (,var)
-                           (unwind-protect
-                               (progn
-                                 (setf ,var (make-local-alien ',info))
-                                 (let ((,var ,var))
-                                   ,@body-forms))
+                       `((let ((,var (make-local-alien ',info)))
+                           (multiple-value-prog1
+                               (progn ,@body-forms)
+                             ;; No need for unwind protect here, since
+                             ;; allocation involves modifying NSP, and
+                             ;; NSP is saved and restored during NLX.
+                             ;; And in non-transformed case it
+                             ;; performs finalization.
                              (dispose-local-alien ',info ,var))))))))))))
     (/show "revised" body)
     (verify-local-auxiliaries-okay)
@@ -196,7 +198,7 @@ This is SETFable."
                         ,(append *new-auxiliary-types*
                                  (auxiliary-type-definitions env))))
        #!+(or x86 x86-64)
-       (let ((sb!vm::*alien-stack* sb!vm::*alien-stack*))
+       (let ((sb!vm::*alien-stack-pointer* sb!vm::*alien-stack-pointer*))
          ,@body)
        #!-(or x86 x86-64)
        ,@body)))
@@ -274,8 +276,7 @@ Examples:
   (type-of *foo*)                 ; => (alien (* (array (signed 8) 10)))
   (setf (deref (deref foo) 0) 10) ; => 10
 
-  (make-alien char 12)            ; => (alien (* (signed 8)))
-"
+  (make-alien char 12)            ; => (alien (* (signed 8)))"
   (let ((alien-type (if (alien-type-p type)
                         type
                         (parse-alien-type type env))))
@@ -362,6 +363,7 @@ allocated by MAKE-ALIEN, MAKE-ALIEN-STRING or malloc(3)."
                                  &key (start 0) end
                                       (external-format :default)
                                       (null-terminate t))
+  #!+sb-doc
   "Copy part of STRING delimited by START and END into freshly
 allocated foreign memory, freeable using free(3) or FREE-ALIEN.
 Returns the allocated string as a (* CHAR) alien, and the number of
@@ -369,8 +371,7 @@ bytes allocated as secondary value.
 
 The string is encoded using EXTERNAL-FORMAT. If NULL-TERMINATE is
 true (the default), the alien string is terminated by an additional
-null byte.
-"
+null byte."
   (declare (ignore start end external-format null-terminate))
   (multiple-value-bind (sap bytes)
       (apply #'%make-alien-string string rest)
@@ -596,6 +597,9 @@ null byte.
     (error "~S isn't forced to memory. Something went wrong." alien))
   alien)
 
+;; It's not mandatory that this function not exist for x86[-64],
+;; however for sanity, it should not, because no call to it can occur.
+#!-(or x86 x86-64)
 (defun dispose-local-alien (info alien)
   (declare (ignore info))
   (cancel-finalization alien)
@@ -838,6 +842,7 @@ way that the argument is passed.
 ;;;; See "Foreign Linkage / Callbacks" in the SBCL Internals manual.
 
 (defvar *alien-callback-info* nil
+  #!+sb-doc
   "Maps SAPs to corresponding CALLBACK-INFO structures: contains all the
 information we need to manipulate callbacks after their creation. Used for
 changing the lisp-side function they point to, invalidation, etc.")
@@ -855,15 +860,18 @@ changing the lisp-side function they point to, invalidation, etc.")
   (cdr (assoc (alien-sap alien) *alien-callback-info* :test #'sap=)))
 
 (defvar *alien-callbacks* (make-hash-table :test #'equal)
+  #!+sb-doc
   "Cache of existing callback SAPs, indexed with (SPECIFER . FUNCTION). Used for
 memoization: we don't create new callbacks if one pointing to the correct
 function with the same specifier already exists.")
 
 (defvar *alien-callback-wrappers* (make-hash-table :test #'equal)
+  #!+sb-doc
   "Cache of existing lisp wrappers, indexed with SPECIFER. Used for memoization:
 we don't create new wrappers if one for the same specifier already exists.")
 
 (defvar *alien-callback-trampolines* (make-array 32 :fill-pointer 0 :adjustable t)
+  #!+sb-doc
   "Lisp trampoline store: assembler wrappers contain indexes to this, and
 ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
 
@@ -959,13 +967,11 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
   (declare (ignore arguments))
   (error "Invalid alien callback called."))
 
-
 (defun parse-callback-specification (result-type lambda-list)
   (values
    `(function ,result-type ,@(mapcar #'second lambda-list))
    (mapcar #'first lambda-list)))
 
-
 (defun parse-alien-ftype (specifier env)
   (destructuring-bind (function result-type &rest argument-types)
       specifier
@@ -1010,6 +1016,7 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
 ;;;; interface (not public, yet) for alien callbacks
 
 (defmacro alien-callback (specifier function &environment env)
+  #!+sb-doc
   "Returns an alien-value with of alien ftype SPECIFIER, that can be passed to
 an alien function as a pointer to the FUNCTION. If a callback for the given
 SPECIFIER and FUNCTION already exists, it is returned instead of consing a new
@@ -1030,6 +1037,7 @@ one."
       ',(parse-alien-type specifier env))))
 
 (defun alien-callback-p (alien)
+  #!+sb-doc
   "Returns true if the alien is associated with a lisp-side callback,
 and a secondary return value of true if the callback is still valid."
   (let ((info (alien-callback-info alien)))
@@ -1037,12 +1045,14 @@ and a secondary return value of true if the callback is still valid."
       (values t (and (callback-info-function info) t)))))
 
 (defun alien-callback-function (alien)
+  #!+sb-doc
   "Returns the lisp function designator associated with the callback."
   (let ((info (alien-callback-info alien)))
     (when info
       (callback-info-function info))))
 
 (defun (setf alien-callback-function) (function alien)
+  #!+sb-doc
   "Changes the lisp function designated by the callback."
   (let ((info (alien-callback-info alien)))
     (unless info
@@ -1059,6 +1069,7 @@ and a secondary return value of true if the callback is still valid."
     function))
 
 (defun invalidate-alien-callback (alien)
+  #!+sb-doc
   "Invalidates the callback designated by the alien, if any, allowing the
 associated lisp function to be GC'd, and causing further calls to the same
 callback signal an error."
@@ -1090,6 +1101,7 @@ callback signal an error."
 ;;; the FDEFINITION should invalidate the callback, and redefining the
 ;;; callback should change existing callbacks to point to the new defintion.
 (defmacro define-alien-callback (name result-type typed-lambda-list &body forms)
+  #!+sb-doc
   "Defines #'NAME as a function with the given body and lambda-list, and NAME as
 the alien callback for that function with the given alien type."
   (declare (symbol name))
