@@ -21,13 +21,19 @@
                      (when (muffle-warning-p warning)
                        (muffle-warning warning)))))))
 
-;;; an alist with elements of the form
+;;; Each cluster is an alist of the form
 ;;;
-;;;  (CONDITION . (HANDLER1 HANDLER2 ...))
+;;;  ((CONDITION-TYPE1 . HANDLER1) (CONDITION-TYPE2 . HANDLER2) ...)
 ;;;
-;;; Recently established handlers are added at the beginning of the
+;;; where CONDITION-TYPEN are type specifiers and HANDLERN are
+;;; function designators.
+;;;
+;;; Newly established handlers are added at the beginning of the
 ;;; list. Elements to the left of the alist take precedence over
 ;;; elements to the right.
+;;;
+;;; Lists to which *HANDLER-CLUSTERS* is bound generally have dynamic
+;;; extent.
 (defvar *handler-clusters* (initial-handler-clusters))
 
 ;;; a list of lists of currently active RESTART instances. maintained
@@ -84,10 +90,8 @@
 ;; 2) and for which the restart test returns non-NIL for CONDITION.
 ;; When CALL-TEST-P is non-NIL, all restarts are processed.
 (defun map-restarts (function &optional condition (call-test-p t))
-  ;; FIXME: if MAP-RESTARTS is internal, we could require the FUNCTION
-  ;; argument to be of type FUNCTION.
-  (let ((function (coerce function 'function))
-        (stack *restart-test-stack*))
+  (declare (function function))
+  (let ((stack *restart-test-stack*))
     (dolist (restart-cluster *restart-clusters*)
       (dolist (restart restart-cluster)
         (when (and (or (not condition)
@@ -115,19 +119,16 @@ restarts associated with CONDITION (or with no condition) will be returned."
     (map-restarts (lambda (restart) (result restart)) condition)
     (result)))
 
-(defun %find-restart (identifier &optional condition (call-test-p t))
+(defun %find-restart (identifier condition &optional (call-test-p t))
   (flet ((eq-restart-p (restart)
            (when (eq identifier restart)
              (return-from %find-restart restart)))
          (named-restart-p (restart)
            (when (eq identifier (restart-name restart))
              (return-from %find-restart restart))))
-    ;; TODO Question for reviewer: does the compiler infer this dx
-    ;; automatically?
+    ;; KLUDGE: can the compiler infer this dx automatically?
     (declare (truly-dynamic-extent #'eq-restart-p #'named-restart-p))
     (if (typep identifier 'restart)
-        ;; TODO Questions for reviewer:
-        ;;
         ;; The code under #+previous-... below breaks the abstraction
         ;; introduced by MAP-RESTARTS, but is about twice as
         ;; fast as #+equivalent-... . Also, it is a common case due to
@@ -216,9 +217,13 @@ with that condition (or with no condition) will be returned."
   "Calls the function associated with the given restart, prompting for any
    necessary arguments. If the argument restart is not a restart or a
    currently active non-NIL restart name, then a CONTROL-ERROR is signalled."
-  (let* ((real-restart (find-restart-or-control-error restart))
+  ;; For an explanation of the call to FIND-RESTART-OR-CONTROL-ERROR,
+  ;; see comment in INVOKE-RESTART.
+  (let* ((real-restart (find-restart-or-control-error
+                        restart nil (symbolp restart)))
          (args (interactive-restart-arguments real-restart)))
     (apply (restart-function real-restart) args)))
+
 
 (defun assert-error (assertion args-and-values places datum &rest arguments)
   (let ((cond (if datum
