@@ -23,11 +23,6 @@
 
 (in-package "SB!IMPL")
 
-(!begin-collecting-cold-init-forms)
-
-(!cold-init-forms
-  (/show0 "entering !PACKAGE-COLD-INIT"))
-
 ;;;; Thread safety
 ;;;;
 ;;;; ...this could still use work, but the basic idea is:
@@ -54,8 +49,6 @@
 ;;;; needs to be considered with some care.
 
 (defvar *package-graph-lock*)
-(!cold-init-forms
- (setf *package-graph-lock* (sb!thread:make-mutex :name "Package Graph Lock")))
 
 (defun call-with-package-graph (function)
   (declare (function function))
@@ -67,8 +60,6 @@
 ;;; a map from package names to packages
 (defvar *package-names*)
 (declaim (type hash-table *package-names*))
-(!cold-init-forms
- (setf *package-names* (make-hash-table :test 'equal :synchronized t)))
 
 (defmacro with-package-names ((names &key) &body body)
   `(let ((,names *package-names*))
@@ -324,11 +315,8 @@ error if any of PACKAGES is not a valid package designator."
 
 (def!method print-object ((package package) stream)
   (let ((name (package-%name package)))
-    (if name
-        (print-unreadable-object (package stream :type t)
-          (prin1 name stream))
-        (print-unreadable-object (package stream :type t :identity t)
-          (write-string "(deleted)" stream)))))
+    (print-unreadable-object (package stream :type t :identity (not name))
+      (if name (prin1 name stream) (write-string "(deleted)" stream)))))
 
 ;;; ANSI says (in the definition of DELETE-PACKAGE) that these, and
 ;;; most other operations, are unspecified for deleted packages. We
@@ -528,22 +516,6 @@ Experimental: interface subject to change."
 ;;; FIXME: should be declared of type PACKAGE, with no NIL init form,
 ;;; after I get around to cleaning up DOCUMENTATION
 
-;;; This magical variable is T during initialization so that
-;;; USE-PACKAGE's of packages that don't yet exist quietly win. Such
-;;; packages are thrown onto the list *DEFERRED-USE-PACKAGES* so that
-;;; this can be fixed up later.
-;;;
-;;; FIXME: This could be cleaned up the same way I do it in my package
-;;; hacking when setting up the cross-compiler. Then we wouldn't have
-;;; this extraneous global variable and annoying runtime tests on
-;;; package operations. (*DEFERRED-USE-PACKAGES* would also go away.)
-(defvar *in-package-init*)
-
-;;; pending USE-PACKAGE arguments saved up while *IN-PACKAGE-INIT* is true
-(defvar *!deferred-use-packages*)
-(!cold-init-forms
-  (setf *!deferred-use-packages* nil))
-
 (define-condition bootstrap-package-not-found (condition)
   ((name :initarg :name :reader bootstrap-package-name)))
 (defun debootstrap-package (&optional condition)
@@ -555,7 +527,7 @@ Experimental: interface subject to change."
   "If PACKAGE-DESIGNATOR is a package, it is returned. Otherwise PACKAGE-DESIGNATOR
 must be a string designator, in which case the package it names is located and returned.
 
-As an SBCL extension, the current package may effect the way a package name is
+As an SBCL extension, the current package may affect the way a package name is
 resolved: if the current package has local nicknames specified, package names
 matching those are resolved to the packages associated with them instead.
 
@@ -584,7 +556,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                                (cdr (assoc string nicknames :test #'string=))))
                   (packageoid (or nicknamed (gethash string *package-names*))))
              (if (and (null packageoid)
-                      (not *in-package-init*) ; KLUDGE
+                      ;; FIXME: should never need 'debootstrap' hack
                       (let ((mismatch (mismatch "SB!" string)))
                         (and mismatch (= mismatch 3))))
                  (restart-case
@@ -607,9 +579,8 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; Return a list of packages given a package designator or list of
 ;;; package designators, or die trying.
 (defun package-listify (thing)
-  (let ((res ()))
-    (dolist (thing (if (listp thing) thing (list thing)) res)
-      (push (find-undeleted-package-or-lose thing) res))))
+  (mapcar #'find-undeleted-package-or-lose
+          (if (listp thing) thing (list thing))))
 
 ;;; Make a package name into a simple-string.
 (defun package-namify (n)
@@ -662,6 +633,10 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
          (if (zerop (the fixnum (aref hash i)))
              (decf (package-hashtable-free table))
              (decf (package-hashtable-deleted table)))
+         ;; This order is critical but the code is not strong enough as is.
+         ;; There should be a write barrier in here somewhere, otherwise
+         ;; in a system which allows out-of-order memory writes,
+         ;; another thread could see a valid hash but 0 for the symbol.
          (setf (svref vec i) symbol)
          (setf (aref hash i)
                (entry-hash (length (symbol-name symbol))
@@ -688,6 +663,30 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; is bound to the index, or NIL if it is not present. SYMBOL-VAR
 ;;; is bound to the symbol. LENGTH and HASH are the length and sxhash
 ;;; of STRING. ENTRY-HASH is the entry-hash of the string and length.
+
+#| WITH-SYMBOL is supposedly thread-safe for single-writer/multi-reader,
+   but it can crash in two ways during resize of a package-hashtable:
+   - MAKE-OR-REMAKE-PACKAGE-HASHTABLE assigns the TABLE slot before
+     the HASH slot, so the initial index computed into HASH can be wrong.
+   - If a hash-vector element is a positive hit but there is a zero in the
+     corresponding location in the symbol-vector, it calls (SYMBOL-NAME 0).
+Example:
+* (let ((h (sxhash "GG"))) (values (mod h 17) (entry-hash 2 h))) => 8, 238
+* (let ((h (sxhash "$o"))) (values (mod h 37) (entry-hash 2 h))) => 8, 238
+* (intern "GG" (make-package "X"))
+;; Simulate growth of TABLE to the next larger size. Also suppose that the
+;; the rehashing thread is switched out by the OS after this setf.
+* (setf (package-hashtable-table (package-internal-symbols (find-package "X")))
+        (make-array 37)) ; the next size up
+* (find-symbol "A" "X") ; failure mode 1 - bad index computation
+debugger invoked on a SB-INT:INVALID-ARRAY-INDEX-ERROR:
+* (find-symboL "$o" "X") ; failure mode 2 - hash matched but no symbol present
+debugger invoked on a TYPE-ERROR: The value 0 is not of type SYMBOL.
+
+Probably we need to atomically store both vectors by keeping them in a cons
+which is freshly allocated whenever the vectors are reallocated.
+It might also work to store the zero-filled hash vector first. |#
+
 (defmacro with-symbol ((index-var symbol-var table string length sxhash
                                   entry-hash)
                        &body forms)
@@ -809,9 +808,7 @@ implementation it is ~S." *default-package-use-list*)
 
          ;; Do a USE-PACKAGE for each thing in the USE list so that checking for
          ;; conflicting exports among used packages is done.
-         (if *in-package-init*
-             (push (list use package) *!deferred-use-packages*)
-             (use-package use package))
+         (use-package use package)
 
          ;; FIXME: ENTER-NEW-NICKNAMES can fail (ERROR) if nicknames are illegal,
          ;; which would leave us with possibly-bad side effects from the earlier
@@ -928,7 +925,7 @@ implementation it is ~S." *default-package-use-list*)
                           ;; for tidiness and to help the GC.
                           (package-%nicknames package) nil))
                   (setf (package-%use-list package) nil
-                        (package-tables package) nil
+                        (package-tables package) #()
                         (package-%shadowing-symbols package) nil
                         (package-internal-symbols package)
                         (make-or-remake-package-hashtable 0)
@@ -947,34 +944,48 @@ implementation it is ~S." *default-package-use-list*)
                names))
     res))
 
-(defun intern (name &optional (package (sane-package)))
+(macrolet ((find/intern (function)
+             ;; Both FIND-SYMBOL* and INTERN* require a SIMPLE-STRING,
+             ;; but accept a LENGTH. Given a non-simple string,
+             ;; we need copy it only if the cumulative displacement
+             ;; into the underlying simple-string is nonzero.
+             ;; There are two things that can be improved
+             ;; about the generated code here:
+             ;; 1. if X is known to satisfy STRINGP (generally any rank-1 array),
+             ;;    then testing SIMPLE-<base|character>-STRING-P should not
+             ;;    re-test the lowtag. This is constrained by the backends,
+             ;;    because there are no type vops that assume a known lowtag.
+             ;; 2. if X is known to satisfy VECTORP, then
+             ;;    (NOT (ARRAY-HEADER-P)) implies SIMPLE-P, but the compiler
+             ;;    does not actually know that, and generates a check.
+             ;;    This is more of a front-end issue.
+             `(multiple-value-bind (name length)
+                  (if (simple-string-p name)
+                      (values name (length name))
+                      (with-array-data ((name name) (start) (end)
+                                        :check-fill-pointer t)
+                        (if (eql start 0)
+                            (values name end)
+                            (values (subseq name start end)
+                                    (- end start)))))
+                (truly-the
+                 (values symbol (member :internal :external :inherited nil))
+                 (,function name length
+                            (find-undeleted-package-or-lose package))))))
+
+  (defun intern (name &optional (package (sane-package)))
   #!+sb-doc
   "Return a symbol in PACKAGE having the specified NAME, creating it
   if necessary."
-  ;; We just simple-stringify the name and call INTERN*, where the real
-  ;; logic is.
-  (let ((name (if (simple-string-p name)
-                  name
-                  (coerce name 'simple-string)))
-        (package (find-undeleted-package-or-lose package)))
-    (declare (simple-string name))
-      (intern* name
-               (length name)
-               package)))
+    (find/intern intern*))
 
-(defun find-symbol (name &optional (package (sane-package)))
+  (defun find-symbol (name &optional (package (sane-package)))
   #!+sb-doc
   "Return the symbol named STRING in PACKAGE. If such a symbol is found
   then the second value is :INTERNAL, :EXTERNAL or :INHERITED to indicate
   how the symbol is accessible. If no symbol is found then both values
   are NIL."
-  ;; We just simple-stringify the name and call FIND-SYMBOL*, where the
-  ;; real logic is.
-  (let ((name (if (simple-string-p name) name (coerce name 'simple-string))))
-    (declare (simple-string name))
-    (find-symbol* name
-                  (length name)
-                  (find-undeleted-package-or-lose package))))
+    (find/intern find-symbol*)))
 
 ;;; If the symbol named by the first LENGTH characters of NAME doesn't exist,
 ;;; then create it, special-casing the keyword package.
@@ -995,6 +1006,8 @@ implementation it is ~S." *default-package-use-list*)
                  (let ((symbol-name (cond (no-copy
                                            (aver (= (length name) length))
                                            name)
+                                          ((typep name '(simple-array nil (*)))
+                                           "")
                                           (t
                                            ;; This so that SUBSEQ is inlined,
                                            ;; because we need it fixed for cold init.
@@ -1032,27 +1045,25 @@ implementation it is ~S." *default-package-use-list*)
                         string length hash ehash)
       (when found
         (return-from find-symbol* (values symbol :external))))
-    (let ((head (package-tables package)))
-      (do ((prev head table)
-           (table (cdr head) (cdr table)))
-          ((null table) (values nil nil))
-        (with-symbol (found symbol (car table) string length hash ehash)
-          (when found
-            ;; At this point we used to move the table to the
-            ;; beginning of the list, probably on the theory that we'd
-            ;; soon be looking up further items there. Unfortunately
-            ;; that was very much non-thread safe. Since the failure
-            ;; mode was nasty (corruption of the package in a way
-            ;; which would make symbol lookups loop infinitely) and it
-            ;; would be triggered just by doing reads to a resource
-            ;; that users can't do their own locking on, that code has
-            ;; been removed. If we ever add locking to packages,
-            ;; resurrecting that code might make sense, even though it
-            ;; didn't seem to have much of an performance effect in
-            ;; normal use.
-            ;;
-            ;; -- JES, 2006-09-13
-            (return-from find-symbol* (values symbol :inherited))))))))
+    (let* ((tables (package-tables package))
+           (n (length tables)))
+      (unless (eql n 0)
+        ;; Try the most-recently-used table, then others.
+        ;; TABLES is treated as circular for this purpose.
+        (let* ((mru (package-mru-table-index package))
+               (start (if (< mru n) mru 0))
+               (i start))
+          (loop
+             (with-symbol (found symbol
+                                 (locally (declare (optimize (safety 0)))
+                                   (svref tables i))
+                                 string length hash ehash)
+               (when found
+                 (setf (package-mru-table-index package) i)
+                 (return-from find-symbol* (values symbol :inherited))))
+             (if (< (decf i) 0) (setq i (1- n)))
+             (if (= i start) (return)))))))
+  (values nil nil))
 
 ;;; Similar to FIND-SYMBOL, but only looks for an external symbol.
 ;;; This is used for fast name-conflict checking in this file and symbol
@@ -1257,24 +1268,6 @@ uninterned."
   (mapcar #'string (if (listp thing)
                        thing
                        (list thing))))
-
-;;; This is like UNINTERN, except if SYMBOL is inherited, it chases
-;;; down the package it is inherited from and uninterns it there. Used
-;;; for name-conflict resolution. Shadowing symbols are not uninterned
-;;; since they do not cause conflicts.
-(defun moby-unintern (symbol package)
-  (unless (member symbol (package-%shadowing-symbols package))
-    (or (unintern symbol package)
-        (let ((name (symbol-name symbol)))
-          (multiple-value-bind (s w) (find-symbol name package)
-            (declare (ignore s))
-            (when (eq w :inherited)
-              (dolist (q (package-%use-list package))
-                (multiple-value-bind (u x) (find-external-symbol name q)
-                  (declare (ignore u))
-                  (when x
-                    (unintern symbol q)
-                    (return t))))))))))
 
 (defun export (symbols &optional (package (sane-package)))
   #!+sb-doc
@@ -1285,10 +1278,9 @@ uninterned."
           (syms ()))
       ;; Punt any symbols that are already external.
       (dolist (sym symbols)
-        (multiple-value-bind (s w)
+        (multiple-value-bind (s found)
             (find-external-symbol (symbol-name sym) package)
-          (declare (ignore s))
-          (unless (or w (member sym syms))
+          (unless (or (and found (eq s sym)) (member sym syms))
             (push sym syms))))
       (with-single-package-locked-error ()
         (when syms
@@ -1515,7 +1507,11 @@ PACKAGE."
                        (name-conflict package 'use-package pkg sym s)))))))
 
             (push pkg (package-%use-list package))
-            (push (package-external-symbols pkg) (cdr (package-tables package)))
+            (setf (package-tables package)
+                  (let ((tbls (package-tables package)))
+                    (replace (make-array (1+ (length tbls))
+                              :initial-element (package-external-symbols pkg))
+                             tbls)))
             (push package (package-%used-by-list pkg)))))))
   t)
 
@@ -1534,7 +1530,7 @@ PACKAGE."
                 (remove p (the list (package-%use-list package))))
           (setf (package-tables package)
                 (delete (package-external-symbols p)
-                        (the list (package-tables package))))
+                        (package-tables package)))
           (setf (package-%used-by-list p)
                 (remove package (the list (package-%used-by-list p))))))
       t)))
@@ -1602,45 +1598,45 @@ PACKAGE."
 
 ;;;; final initialization
 
-;;;; The cold loader (GENESIS) makes the data structure in
-;;;; *!INITIAL-SYMBOLS*. We grovel over it, making the specified
-;;;; packages and interning the symbols. For a description of the
-;;;; format of *!INITIAL-SYMBOLS*, see the GENESIS source.
-
+;;;; Due to the relative difficulty - but not impossibility - of manipulating
+;;;; package-hashtables in the cross-compilation host, all interning operations
+;;;; are delayed until cold-init.
+;;;; The cold loader (GENESIS) set *!INITIAL-SYMBOLS* to the target
+;;;; representation of the hosts's *COLD-PACKAGE-SYMBOLS*.
+;;;; The shape of this list is ((package . (externals . internals)) ...)
 (defvar *!initial-symbols*)
 
-(!cold-init-forms
+(defun !package-cold-init ()
+  (setf *package-graph-lock* (sb!thread:make-mutex :name "Package Graph Lock")
+        *package-names* (make-hash-table :test 'equal :synchronized t))
+  (with-package-names (names)
+    (dolist (spec *!initial-symbols*)
+      (let ((pkg (car spec)) (symbols (cdr spec)))
+        ;; the symbol MAKE-TABLE wouldn't magically disappear,
+        ;; though its only use be to name an FLET in a function
+        ;; hanging on an otherwise uninternable symbol. strange but true :-(
+        (flet ((!make-table (input)
+                 (let ((table (make-or-remake-package-hashtable
+                               (length (the simple-vector input)))))
+                   (dovector (symbol input table)
+                     (add-symbol table symbol)))))
+          (setf (package-external-symbols pkg) (!make-table (car symbols))
+                (package-internal-symbols pkg) (!make-table (cdr symbols))))
+        (setf (package-%shadowing-symbols pkg) nil
+              (package-%local-nicknames pkg) nil
+              (package-%locally-nicknamed-by pkg) nil
+              (package-source-location pkg) nil
+              (gethash (package-%name pkg) names) pkg)
+        (%enter-new-nicknames pkg (package-%nicknames pkg))
+        #!+sb-package-locks
+        (setf (package-lock pkg) nil
+              (package-%implementation-packages pkg) nil))))
 
-  (setq *in-package-init* t)
-
-  (/show0 "about to loop over *!INITIAL-SYMBOLS* to make packages")
+  ;; pass 2 - set the 'tables' slots only after all tables have been made
   (dolist (spec *!initial-symbols*)
-    (let* ((pkg (apply #'make-package (first spec)))
-           (internal (package-internal-symbols pkg))
-           (external (package-external-symbols pkg)))
-      (/show0 "back from MAKE-PACKAGE, PACKAGE-NAME=..")
-      (/primitive-print (package-name pkg))
-
-      ;; Put internal symbols in the internal hashtable and set package.
-      (dolist (symbol (second spec))
-        (add-symbol internal symbol)
-        (%set-symbol-package symbol pkg))
-
-      ;; External symbols same, only go in external table.
-      (dolist (symbol (third spec))
-        (add-symbol external symbol)
-        (%set-symbol-package symbol pkg))
-
-      ;; Don't set package for imported symbols.
-      (dolist (symbol (fourth spec))
-        (add-symbol internal symbol))
-      (dolist (symbol (fifth spec))
-        (add-symbol external symbol))
-
-      ;; Put shadowing symbols in the shadowing symbols list.
-      (setf (package-%shadowing-symbols pkg) (sixth spec))
-      ;; Set the package documentation
-      (setf (package-doc-string pkg) (seventh spec))))
+    (let ((pkg (car spec)))
+      (setf (package-tables pkg)
+            (map 'vector #'package-external-symbols (package-%use-list pkg)))))
 
   ;; FIXME: These assignments are also done at toplevel in
   ;; boot-extensions.lisp. They should probably only be done once.
@@ -1669,23 +1665,9 @@ PACKAGE."
                        "SB!ALIEN" "SB!ALIEN" "SB!DEBUG"
                        "SB!EXT" "SB!GRAY" "SB!PROFILE"))
 
-  ;; Now do the *!DEFERRED-USE-PACKAGES*.
-  (/show0 "about to do *!DEFERRED-USE-PACKAGES*")
-  (dolist (args *!deferred-use-packages*)
-    (apply #'use-package args))
-
-  ;; The Age Of Magic is over, we can behave ANSIly henceforth.
-  (/show0 "about to SETQ *IN-PACKAGE-INIT*")
-  (setq *in-package-init* nil)
-
   ;; For the kernel core image wizards, set the package to *CL-PACKAGE*.
   ;;
   ;; FIXME: We should just set this to (FIND-PACKAGE
   ;; "COMMON-LISP-USER") once and for all here, instead of setting it
   ;; once here and resetting it later.
   (setq *package* *cl-package*))
-
-(!cold-init-forms
-  (/show0 "done with !PACKAGE-COLD-INIT"))
-
-(!defun-from-collected-cold-init-forms !package-cold-init)
