@@ -116,8 +116,8 @@
          ;; FIND-CLASSOID caused suboptimal register allocation in PCL
          ;; dfuns. So instead we now use a special variable which is
          ;; initialized during cold init. -- JES, 2006-07-04
-         *null-classoid-layout*)
-        (t (svref *built-in-class-codes* (widetag-of x)))))
+         **null-classoid-layout**)
+        (t (svref **built-in-class-codes** (widetag-of x)))))
 
 #!-sb-fluid (declaim (inline classoid-of))
 (defun classoid-of (object)
@@ -143,10 +143,7 @@
 ;;; type checking, rather than trying to come up with the one that the
 ;;; user might find most informative.
 (declaim (ftype (function (t) ctype) ctype-of))
-(defun-cached (ctype-of
-               :hash-function (lambda (x) (logand (sxhash x) #x1FF))
-               :hash-bits 9
-               :init-wrapper !cold-init-forms)
+(defun-cached (ctype-of :hash-function #'sxhash :hash-bits 9)
               ((x eq))
   (typecase x
     (function
@@ -159,7 +156,7 @@
      (ctype-of-number x))
     (array
      (let ((etype (specifier-type (array-element-type x))))
-       (make-array-type :dimensions (array-dimensions x)
+       (make-array-type (array-dimensions x)
                         :complexp (not (typep x 'simple-array))
                         :element-type etype
                         :specialized-element-type etype)))
