@@ -98,6 +98,8 @@
 ;;; This is not FLUSHABLE, since it's required to signal an error if
 ;;; unbound.
 (defknown (symbol-value) (symbol) t ())
+(defknown about-to-modify-symbol-value (symbol t &optional t t) null
+  (explicit-check))
 ;;; From CLHS, "If the symbol is globally defined as a macro or a
 ;;; special operator, an object of implementation-dependent nature and
 ;;; identity is returned. If the symbol is not globally defined as
@@ -216,6 +218,9 @@
 (defknown (use-package unuse-package)
   ((or list package-designator) &optional package-designator) (eql t))
 (defknown find-all-symbols (string-designator) list (flushable))
+;; private
+(defknown package-iter-step (fixnum index t simple-vector list)
+  (values fixnum index t simple-vector list symbol symbol))
 
 ;;;; from the "Numbers" chapter:
 
@@ -468,12 +473,12 @@
 (defknown copy-seq (sequence) consed-sequence (flushable)
   :derive-type (sequence-result-nth-arg 1))
 
-(defknown length (sequence) index (foldable flushable dx-safe))
+(defknown length (sequence) index (explicit-check foldable flushable dx-safe))
 
-(defknown reverse (sequence) consed-sequence (flushable)
+(defknown reverse (sequence) consed-sequence (explicit-check flushable)
   :derive-type (sequence-result-nth-arg 1))
 
-(defknown nreverse (sequence) sequence (important-result)
+(defknown nreverse (sequence) sequence (explicit-check important-result)
   :derive-type #'result-type-first-arg
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
@@ -498,6 +503,7 @@
   (call)
 ; :DERIVE-TYPE 'TYPE-SPEC-ARG1 ? Nope... (MAP NIL ...) returns NULL, not NIL.
   )
+(defknown %map-for-effect-arity-1 (callable sequence) null (call))
 (defknown %map-to-list-arity-1 (callable sequence) list (flushable call))
 (defknown %map-to-simple-vector-arity-1 (callable sequence) simple-vector
   (flushable call))
@@ -629,13 +635,15 @@
                     (:test-not callable) (:start index) (:from-end t)
                     (:end sequence-end) (:key callable))
   (or index null)
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'position-derive-type)
 
 (defknown (position-if position-if-not)
   (callable sequence &rest t &key (:from-end t) (:start index)
    (:end sequence-end) (:key callable))
   (or index null)
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'position-derive-type)
 
 (defknown count (t sequence &rest t &key
                    (:test callable) (:test-not callable) (:start index)
@@ -742,14 +750,10 @@
 (defknown make-list (index &key (:initial-element t)) list
   (movable flushable))
 
-(defknown sb!impl::backq-list (&rest t) list (movable flushable))
-(defknown sb!impl::backq-list* (t &rest t) t (movable flushable))
-(defknown sb!impl::backq-append (&rest t) t (flushable))
-(defknown sb!impl::backq-nconc (&rest t) t ()
-  :destroyed-constant-args (remove-non-constants-and-nils #'butlast))
-(defknown sb!impl::backq-cons (t t) cons (foldable movable flushable))
-(defknown sb!impl::backq-vector (list) simple-vector
-    (foldable movable flushable))
+(defknown sb!impl::|List| (&rest t) list (movable flushable foldable))
+(defknown sb!impl::|List*| (t &rest t) t (movable flushable foldable))
+(defknown sb!impl::|Append| (&rest t) t (flushable foldable))
+(defknown sb!impl::|Vector| (&rest t) simple-vector (flushable foldable))
 
 ;;; All but last must be of type LIST, but there seems to be no way to
 ;;; express that in this syntax.
@@ -766,8 +770,6 @@
 ;;; not check it now :-).
 (defknown nconc (&rest t) t ()
   :destroyed-constant-args (remove-non-constants-and-nils #'butlast))
-(defknown sb!impl::nconc2 (list t) t ()
-  :destroyed-constant-args (remove-non-constants-and-nils #'butlast))
 
 (defknown nreconc (list t) t (important-result)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
@@ -887,7 +889,9 @@
                       (:initial-element t)
                       (:initial-contents t)
                       (:adjustable t)
-                      (:fill-pointer t)
+                      ;; the type constraint doesn't do anything
+                      ;; on account of EXPLICIT-CHECK. it's documentation.
+                      (:fill-pointer (or index boolean))
                       (:displaced-to (or array null))
                       (:displaced-index-offset index))
   array (flushable explicit-check))
@@ -900,7 +904,7 @@
                        (:initial-element t)
                        (:initial-contents t)
                        (:adjustable t)
-                       (:fill-pointer t)
+                       (:fill-pointer (or index boolean))
                        (:displaced-to (or array null))
                        (:displaced-index-offset index))
     array (flushable))
@@ -914,6 +918,10 @@
   type-specifier
   (foldable flushable recursive))
 (defknown array-rank (array) array-rank (foldable flushable))
+;; FIXME: there's a fencepost bug, but for all practical purposes our
+;; ARRAY-RANK-LIMIT is infinite, thus masking the bug. e.g. if the
+;; exclusive limit on rank were 8, then your dimension numbers can
+;; be in the range 0 through 6, not 0 through 7.
 (defknown array-dimension (array array-rank) index (foldable flushable))
 (defknown array-dimensions (array) list (foldable flushable))
 (defknown array-in-bounds-p (array &rest integer) boolean (foldable flushable))
@@ -964,7 +972,8 @@
 (defknown adjust-array
   (array (or index list) &key (:element-type type-specifier)
          (:initial-element t) (:initial-contents t)
-         (:fill-pointer t) (:displaced-to (or array null))
+         (:fill-pointer (or index boolean))
+         (:displaced-to (or array null))
          (:displaced-index-offset index))
   array ())
 ;  :derive-type 'result-type-arg1) Not even close...
@@ -1118,19 +1127,28 @@
   ())
 
 ;;; may return any type due to eof-value...
-(defknown (read read-preserving-whitespace read-char-no-hang read-char)
+;;; and because READ generally returns anything.
+(defknown (read read-preserving-whitespace)
   (&optional stream-designator t t t) t (explicit-check))
 
+(defknown read-char (&optional stream-designator t t t) t (explicit-check)
+  :derive-type (read-elt-type-deriver nil 'character nil))
+(defknown read-char-no-hang (&optional stream-designator t t t) t
+  (explicit-check)
+  :derive-type (read-elt-type-deriver nil 'character t))
+
 (defknown read-delimited-list (character &optional stream-designator t) list
   (explicit-check))
+;; FIXME: add a type-deriver => (values (or string eof-value) boolean)
 (defknown read-line (&optional stream-designator t t t) (values t boolean)
   (explicit-check))
 (defknown unread-char (character &optional stream-designator) t
   (explicit-check))
 (defknown peek-char (&optional (or character (member nil t))
-                               stream-designator t t t)
-  t
-  (explicit-check))
+                               stream-designator t t t) t
+  (explicit-check)
+  :derive-type (read-elt-type-deriver t 'character nil))
+
 (defknown listen (&optional stream-designator) boolean (flushable explicit-check))
 
 (defknown clear-input (&optional stream-designator) null (explicit-check))
@@ -1150,7 +1168,8 @@
           (:junk-allowed t))
   (values (or integer null ()) index))
 
-(defknown read-byte (stream &optional t t) t (explicit-check))
+(defknown read-byte (stream &optional t t) t (explicit-check)
+  :derive-type (read-elt-type-deriver nil 'integer nil))
 
 (defknown (prin1 print princ) (t &optional stream-designator)
   t
@@ -1374,6 +1393,8 @@
 ;;; and analogous SBCL extension:
 (defknown sb!impl::%failed-aver (t) nil)
 (defknown bug (t &rest t) nil) ; never returns
+(defknown sb!int:simple-reader-error (stream string &rest t) nil)
+(defknown sb!kernel:reader-eof-error (stream string) nil)
 
 
 ;;;; from the "Miscellaneous" Chapter:
@@ -1519,7 +1540,7 @@
 (defknown %more-kw-arg (t fixnum) (values t t))
 (defknown %more-arg-values (t index index) * (flushable))
 (defknown %verify-arg-count (index index) (values))
-(defknown %arg-count-error (t) nil)
+(defknown %arg-count-error (t t) nil)
 (defknown %unknown-values () *)
 (defknown %catch (t t) t)
 (defknown %unwind-protect (t t) t)
@@ -1553,6 +1574,8 @@
   (foldable unsafely-flushable explicit-check always-translatable))
 (defknown data-vector-ref-with-offset (simple-array fixnum fixnum) t
   (foldable unsafely-flushable explicit-check always-translatable))
+(defknown data-nil-vector-ref (simple-array index) nil
+  (explicit-check always-translatable))
 (defknown data-vector-set (array index t) t
   (explicit-check always-translatable))
 (defknown data-vector-set-with-offset (array fixnum fixnum t) t
@@ -1655,14 +1678,6 @@
 (defknown %check-vector-sequence-bounds (vector index sequence-end)
   index
   (unwind))
-;;; FIXME: including this information here is probably necessary to
-;;; get efficient compilation of the inline expansion of
-;;; %FIND-POSITION-IF, so it should maybe be in a more
-;;; compiler-friendly package (SB-INT?)
-(defknown sb!impl::signal-bounding-indices-bad-error
-    (sequence index sequence-end)
-  nil) ; never returns
-
 
 (defknown arg-count-error (t t t t t t) nil ())
 
@@ -1771,5 +1786,9 @@
     ())
 (defknown %compare-and-swap-symbol-value (symbol t t) t
     (unwind))
+(defknown (%atomic-dec-symbol-global-value %atomic-inc-symbol-global-value)
+    (symbol fixnum) fixnum)
+(defknown (%atomic-dec-car %atomic-inc-car %atomic-dec-cdr %atomic-inc-cdr)
+    (cons fixnum) fixnum)
 (defknown spin-loop-hint () (values)
     (always-translatable))
