@@ -410,7 +410,10 @@
                         (default-bindings `(,var ,default))
                         (default-vals var))))
                 (let ((bindings (default-bindings))
-                      (call `(%funcall ,fun ,@(reverse vals) ,@(default-vals))))
+                      (call
+                       `(locally
+                            (declare (muffle-conditions code-deletion-note))
+                          (%funcall ,fun ,@(reverse vals) ,@(default-vals)))))
                   (ir1-convert-lambda-body (if bindings
                                                `((let (,@bindings) ,call))
                                                `(,call))
@@ -712,9 +715,8 @@
              (default (arg-info-default info))
              (hairy-default (not (sb!xc:constantp default)))
              (supplied-p (arg-info-supplied-p info))
-             (n-val (make-symbol (format nil
-                                         "~A-DEFAULTING-TEMP"
-                                         (leaf-source-name key))))
+             ;; was: (format nil "~A-DEFAULTING-TEMP" (leaf-source-name key))
+             (n-val (make-symbol ".DEFAULTING-TEMP."))
              (val-temp (make-lambda-var :%source-name n-val)))
         (main-vars val-temp)
         (bind-vars key)
