@@ -11,23 +11,8 @@
 
 (in-package "SB!C")
 
-(declaim (special *wild-type* *universal-type* *compiler-error-context*))
+(declaim (special *compiler-error-context*))
 
-;;; An INLINEP value describes how a function is called. The values
-;;; have these meanings:
-;;;     NIL     No declaration seen: do whatever you feel like, but don't
-;;;             dump an inline expansion.
-;;; :NOTINLINE  NOTINLINE declaration seen: always do full function call.
-;;;    :INLINE  INLINE declaration seen: save expansion, expanding to it
-;;;             if policy favors.
-;;; :MAYBE-INLINE
-;;;             Retain expansion, but only use it opportunistically.
-;;;             :MAYBE-INLINE is quite different from :INLINE. As explained
-;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
-;;;             instantiated once per component, INLINE - for all
-;;;             references (even under #'without FUNCALL)."
-(deftype inlinep () '(member :inline :maybe-inline :notinline nil))
-
 ;;;; source-hacking defining forms
 
 ;;; Parse a DEFMACRO-style lambda-list, setting things up so that a
@@ -277,75 +262,72 @@
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 
 ;;; Given a DEFTRANSFORM-style lambda-list, generate code that parses
-;;; the arguments of a combination with respect to that
-;;; lambda-list. BODY is the list of forms which are to be
-;;; evaluated within the bindings. ARGS is the variable that holds
-;;; list of argument lvars. ERROR-FORM is a form which is evaluated
-;;; when the syntax of the supplied arguments is incorrect or a
-;;; non-constant argument keyword is supplied. Defaults and other gunk
-;;; are ignored. The second value is a list of all the arguments
-;;; bound. We make the variables IGNORABLE so that we don't have to
-;;; manually declare them IGNORE if their only purpose is to make the
-;;; syntax work.
-(defun parse-deftransform (lambda-list body args error-form)
+;;; the arguments of a combination with respect to that lambda-list.
+;;; NODE-VAR the variable that holds the combination node.
+;;; ERROR-FORM is a form which is evaluated when the syntax of the
+;;; supplied arguments is incorrect or a non-constant argument keyword
+;;; is supplied. Defaults and other gunk are ignored. The second value
+;;; is a list of all the arguments bound, which the caller should make
+;;; IGNORABLE if their only purpose is to make the syntax work.
+(defun parse-deftransform (lambda-list node-var error-form)
   (multiple-value-bind (req opt restp rest keyp keys allowp)
       (parse-lambda-list lambda-list)
-    (let* ((min-args (length req))
-           (max-args (+ min-args (length opt)))
-           (n-keys (gensym)))
-      (collect ((binds)
-                (vars)
-                (pos 0 +)
-                (keywords))
-        (dolist (arg req)
-          (vars arg)
-          (binds `(,arg (nth ,(pos) ,args)))
-          (pos 1))
-
+    (let* ((tail (make-symbol "ARGS"))
+           (dummies (make-gensym-list 2))
+           (all-dummies (cons tail dummies))
+           (keys (mapcar (lambda (spec)
+                           (multiple-value-bind (key var)
+                               (if (atom spec)
+                                   (values (keywordicate spec) spec)
+                                   (let ((head (car spec)))
+                                     (if (atom head)
+                                         (values (keywordicate head) head)
+                                         (values (car head) (cadr head)))))
+                             (cons var key)))
+                         keys))
+           final-mandatory-arg)
+      (collect ((binds))
+        (binds `(,tail (basic-combination-args ,node-var)))
+        ;; The way this code checks for mandatory args is to verify that
+        ;; the last positional arg is not null (it should be an LVAR).
+        ;; But somebody could pedantically declare IGNORE on the last arg
+        ;; so bind a dummy for it and then bind from the dummy.
+        (mapl (lambda (args)
+                (cond ((cdr args)
+                       (binds `(,(car args) (pop ,tail))))
+                      (t
+                       (setq final-mandatory-arg (pop dummies))
+                       (binds `(,final-mandatory-arg (pop ,tail))
+                              `(,(car args) ,final-mandatory-arg)))))
+              req)
+        ;; Optionals are pretty easy.
         (dolist (arg opt)
-          (let ((var (if (atom arg) arg (first  arg))))
-            (vars var)
-            (binds `(,var (nth ,(pos) ,args)))
-            (pos 1)))
-
+          (binds `(,(if (atom arg) arg (car arg)) (pop ,tail))))
+        ;; Now if min or max # of args is incorrect,
+        ;; or there are unacceptable keywords, bail out
+        (when (or req keyp (not restp))
+          (binds `(,(pop dummies) ; binding is for effect, not value
+                   (unless (and ,@(if req `(,final-mandatory-arg))
+                                ,@(if (and (not restp) (not keyp))
+                                      `((endp ,tail)))
+                                ,@(if keyp
+                                      (if allowp
+                                          `((check-key-args-constant ,tail))
+                                          `((check-transform-keys
+                                             ,tail ',(mapcar #'cdr keys))))))
+                      ,error-form))))
         (when restp
-          (vars rest)
-          (binds `(,rest (nthcdr ,(pos) ,args))))
-
-        (dolist (spec keys)
-          (if (or (atom spec) (atom (first spec)))
-              (let* ((var (if (atom spec) spec (first spec)))
-                     (key (keywordicate var)))
-                (vars var)
-                (binds `(,var (find-keyword-lvar ,n-keys ,key)))
-                (keywords key))
-              (let* ((head (first spec))
-                     (var (second head))
-                     (key (first head)))
-                (vars var)
-                (binds `(,var (find-keyword-lvar ,n-keys ,key)))
-                (keywords key))))
-
-        (let ((n-length (gensym))
-              (limited-legal (not (or restp keyp))))
-          (values
-           `(let ((,n-length (length ,args))
-                  ,@(when keyp `((,n-keys (nthcdr ,(pos) ,args)))))
-              (unless (and
-                       ;; FIXME: should be PROPER-LIST-OF-LENGTH-P
-                       ,(if limited-legal
-                            `(<= ,min-args ,n-length ,max-args)
-                            `(<= ,min-args ,n-length))
-                       ,@(when keyp
-                           (if allowp
-                               `((check-key-args-constant ,n-keys))
-                               `((check-transform-keys ,n-keys ',(keywords))))))
-                ,error-form)
-              (let ,(binds)
-                (declare (ignorable ,@(vars)))
-                ,@body))
-           (vars)))))))
-
+          (binds `(,rest ,tail)))
+        ;; Return list of bindings, the list of user-specified symbols,
+        ;; and the list of gensyms to be declared ignorable.
+        (values (append (binds)
+                        (mapcar (lambda (k)
+                                  `(,(car k)
+                                     (find-keyword-lvar ,tail ',(cdr k))))
+                                keys))
+                (append (nset-difference (mapcar #'car (binds)) all-dummies)
+                        (mapcar #'car keys))
+                (intersection (mapcar #'car (binds)) (cdr all-dummies)))))))
 ) ; EVAL-WHEN
 
 ;;;; DEFTRANSFORM
@@ -403,45 +385,46 @@
 ;;;             Name with the specified transform definition function. This
 ;;;             may be later instantiated with %DEFTRANSFORM.
 ;;;   :IMPORTANT
-;;;           - If supplied and non-NIL, note this transform as ``important,''
-;;;             which means efficiency notes will be generated when this
-;;;             transform fails even if INHIBIT-WARNINGS=SPEED (but not if
-;;;             INHIBIT-WARNINGS>SPEED).
+;;;           - If the transform fails and :IMPORTANT is
+;;;               NIL,       then never print an efficiency note.
+;;;               :SLIGHTLY, then print a note if SPEED>=INHIBIT-WARNINGS.
+;;;               T,         then print a note if SPEED>INHIBIT-WARNINGS.
+;;;             :SLIGHTLY is the default.
 (defmacro deftransform (name (lambda-list &optional (arg-types '*)
                                           (result-type '*)
                                           &key result policy node defun-only
-                                          eval-name important)
+                                          eval-name (important :slightly))
                              &body body-decls-doc)
+  (declare (type (member nil :slightly t) important))
   (when (and eval-name defun-only)
     (error "can't specify both DEFUN-ONLY and EVAL-NAME"))
   (multiple-value-bind (body decls doc) (parse-body body-decls-doc)
-    (let ((n-args (sb!xc:gensym))
-          (n-node (or node (sb!xc:gensym)))
+    (let ((n-node (or node (sb!xc:gensym)))
           (n-decls (sb!xc:gensym))
-          (n-lambda (sb!xc:gensym))
-          (decls-body `(,@decls ,@body)))
-      (multiple-value-bind (parsed-form vars)
-          (parse-deftransform lambda-list
-                              (if policy
-                                  `((unless (policy ,n-node ,policy)
-                                      (give-up-ir1-transform))
-                                    ,@decls-body)
-                                  body)
-                              n-args
+          (n-lambda (sb!xc:gensym)))
+      (multiple-value-bind (bindings vars)
+          (parse-deftransform lambda-list n-node
                               '(give-up-ir1-transform))
         (let ((stuff
-               `((,n-node)
-                 (let* ((,n-args (basic-combination-args ,n-node))
-                        ,@(when result
-                            `((,result (node-lvar ,n-node)))))
-                   (multiple-value-bind (,n-lambda ,n-decls)
-                       ,parsed-form
+               `((,n-node &aux ,@bindings
+                               ,@(when result
+                                   `((,result (node-lvar ,n-node)))))
+                 (declare (ignorable ,@(mapcar #'car bindings)))
+                 ,@decls
+                 ,@(if policy
+                      `((unless (policy ,n-node ,policy)
+                          (give-up-ir1-transform))))
+                 ;; What purpose does it serve to allow the transform's body
+                 ;; to return decls as a second value? They would go in the
+                 ;; right place if simply returned as part of the expression.
+                 (multiple-value-bind (,n-lambda ,n-decls)
+                       (progn ,@body)
                      (if (and (consp ,n-lambda) (eq (car ,n-lambda) 'lambda))
                          ,n-lambda
                        `(lambda ,',lambda-list
                           (declare (ignorable ,@',vars))
                           ,@,n-decls
-                          ,,n-lambda)))))))
+                          ,,n-lambda))))))
           (if defun-only
               `(defun ,name ,@(when doc `(,doc)) ,@stuff)
               `(%deftransform
@@ -451,7 +434,7 @@
                      `'(function ,arg-types ,result-type))
                 (lambda ,@stuff)
                 ,doc
-                ,(if important t nil))))))))
+                ,important)))))))
 
 ;;;; DEFKNOWN and DEFOPTIMIZER
 
@@ -517,31 +500,45 @@
 ;;; the rest of the optimizer function's lambda-list. LTN-ANNOTATE
 ;;; methods are passed an additional POLICY argument, and IR2-CONVERT
 ;;; methods are passed an additional IR2-BLOCK argument.
-(defmacro defoptimizer (what (lambda-list &optional (n-node (sb!xc:gensym))
-                                          &rest vars)
-                             &body body)
-  (flet ((function-name (name)
-           (etypecase name
-             (symbol name)
-             ((cons (eql setf) (cons symbol null))
-              (symbolicate (car name) "-" (cadr name))))))
-   (let ((name (if (symbolp what)
-                   what
-                   (symbolicate (function-name (first what))
-                                "-" (second what) "-OPTIMIZER"))))
-
-     (let ((n-args (gensym)))
-       `(progn
-          (defun ,name (,n-node ,@vars)
-            (declare (ignorable ,@vars))
-            (let ((,n-args (basic-combination-args ,n-node)))
-              ,(parse-deftransform lambda-list body n-args
-                                   `(return-from ,name nil))))
-          ,@(when (consp what)
-              `((setf (,(let ((*package* (symbol-package 'fun-info)))
+(defmacro defoptimizer (what (lambda-list
+                              &optional (node (sb!xc:gensym) node-p)
+                              &rest vars)
+                        &body body)
+  (binding* ((name
+              (flet ((function-name (name)
+                       (etypecase name
+                         (symbol name)
+                         ((cons (eql setf) (cons symbol null))
+                          (symbolicate (car name) "-" (cadr name))))))
+                (if (symbolp what)
+                    what
+                    (symbolicate (function-name (first what))
+                                 "-" (second what) "-OPTIMIZER"))))
+             ((forms decls) (parse-body body :doc-string-allowed nil))
+             ((var-decls more-decls) (extract-var-decls decls vars))
+             ;; In case the BODY declares IGNORE of the formal NODE var,
+             ;; we rebind it from N-NODE and never reference it from BINDS.
+             (n-node (make-symbol "NODE"))
+             ((binds lambda-vars gensyms)
+              (parse-deftransform lambda-list n-node
+                                  `(return-from ,name nil))))
+    (declare (ignore lambda-vars))
+    `(progn
+       ;; We can't stuff the BINDS as &AUX vars into the lambda list
+       ;; because there can be a RETURN-FROM in there.
+       (defun ,name (,n-node ,@vars)
+         ,@(if var-decls (list var-decls))
+         (let* (,@binds ,@(if node-p `((,node ,n-node))))
+           ;; Syntax requires naming NODE even if undesired if VARS
+           ;; are present, so in that case make NODE ignorable.
+           (declare (ignorable ,@(if (and vars node-p) `(,node))
+                               ,@gensyms))
+           ,@more-decls ,@forms))
+       ,@(when (consp what)
+           `((setf (,(let ((*package* (symbol-package 'fun-info)))
                           (symbolicate "FUN-INFO-" (second what)))
                        (fun-info-or-lose ',(first what)))
-                      #',name))))))))
+                      #',name))))))
 
 ;;;; IR groveling macros
 
@@ -870,52 +867,34 @@
 #!-sb-fluid (declaim (inline find-in position-in))
 
 ;;; Find ELEMENT in a null-terminated LIST linked by the accessor
-;;; function NEXT. KEY, TEST and TEST-NOT are the same as for generic
-;;; sequence functions.
+;;; function NEXT. KEY and TEST are the same as for generic sequence functions.
 (defun find-in (next
                 element
                 list
                 &key
                 (key #'identity)
-                (test #'eql test-p)
-                (test-not #'eql not-p))
-  (declare (type function next key test test-not))
-  (when (and test-p not-p)
-    (error "It's silly to supply both :TEST and :TEST-NOT arguments."))
-  (if not-p
-      (do ((current list (funcall next current)))
-          ((null current) nil)
-        (unless (funcall test-not (funcall key current) element)
-          (return current)))
-      (do ((current list (funcall next current)))
-          ((null current) nil)
-        (when (funcall test (funcall key current) element)
-          (return current)))))
+                (test #'eql))
+  (declare (type function next key test))
+  (do ((current list (funcall next current)))
+      ((null current) nil)
+    (when (funcall test (funcall key current) element)
+      (return current))))
 
 ;;; Return the position of ELEMENT (or NIL if absent) in a
-;;; null-terminated LIST linked by the accessor function NEXT. KEY,
-;;; TEST and TEST-NOT are the same as for generic sequence functions.
+;;; null-terminated LIST linked by the accessor function NEXT.
+;;; KEY and TEST are the same as for generic sequence functions.
 (defun position-in (next
                     element
                     list
                     &key
                     (key #'identity)
-                    (test #'eql test-p)
-                    (test-not #'eql not-p))
-  (declare (type function next key test test-not))
-  (when (and test-p not-p)
-    (error "It's silly to supply both :TEST and :TEST-NOT arguments."))
-  (if not-p
-      (do ((current list (funcall next current))
-           (i 0 (1+ i)))
-          ((null current) nil)
-        (unless (funcall test-not (funcall key current) element)
-          (return i)))
-      (do ((current list (funcall next current))
-           (i 0 (1+ i)))
-          ((null current) nil)
-        (when (funcall test (funcall key current) element)
-          (return i)))))
+                    (test #'eql))
+  (declare (type function next key test))
+  (do ((current list (funcall next current))
+       (i 0 (1+ i)))
+      ((null current) nil)
+    (when (funcall test (funcall key current) element)
+      (return i))))
 
 
 ;;; KLUDGE: This is expanded out twice, by cut-and-paste, in a
