@@ -324,3 +324,11 @@
       (test-type value error t (list-pointer-lowtag) :temp temp)
       (move result value))))
 
+;; A vop that accepts a computed set of widetags.
+(define-vop (%other-pointer-subtype-p type-predicate)
+  (:translate %other-pointer-subtype-p)
+  (:info target not-p widetags)
+  (:arg-types * (:constant t)) ; voodoo - 'target' and 'not-p' are absent
+  (:generator 15 ; arbitrary
+    (%test-headers value target not-p nil (canonicalize-headers widetags)
+                   :temp temp)))
