@@ -275,7 +275,6 @@
   (:vop-var vop)
   (:generator 1
     (emit-alignment n-lowtag-bits)
-    (trace-table-entry trace-table-fun-prologue)
     (emit-label start-lab)
     ;; Skip space for the function header.
     (inst simple-fun-header-word)
@@ -295,9 +294,7 @@
             (make-ea :dword :base ebp-tn
                      :disp (- (* n-word-bytes
                                  (- (max 3 (sb-allocated-size 'stack))
-                                    sp->fp-offset))))))
-
-    (trace-table-entry trace-table-normal)))
+                                    sp->fp-offset))))))))
 
 ;;; This is emitted directly before either a known-call-local, call-local,
 ;;; or a multiple-call-local. All it does is allocate stack space for the
@@ -350,7 +347,7 @@
       ((<= nvals 1)
        (note-this-location vop :single-value-return)
        (cond
-         ((<= (sb!kernel:values-type-max-value-count type)
+         ((<= (values-type-max-value-count type)
               register-arg-count)
           (when (and (named-type-p type)
                      (eq nil (named-type-name type)))
@@ -358,7 +355,7 @@
             ;; ends right here leavig the :SINGLE-VALUE-RETURN note
             ;; dangling. Let's emit a NOP.
             (inst nop)))
-         ((not (sb!kernel:values-type-may-be-single-value-p type))
+         ((not (values-type-may-be-single-value-p type))
           (inst mov esp-tn ebx-tn))
          ((member :cmov *backend-subfeatures*)
           (inst cmov :c esp-tn ebx-tn))
@@ -369,7 +366,7 @@
             (emit-label single-value)))))
       ((<= nvals register-arg-count)
        (note-this-location vop :unknown-return)
-       (when (sb!kernel:values-type-may-be-single-value-p type)
+       (when (values-type-may-be-single-value-p type)
          (let ((regs-defaulted (gen-label)))
            (inst jmp :c regs-defaulted)
            ;; Default the unsupplied registers.
@@ -385,7 +382,7 @@
            (inst mov ebx-tn esp-tn)
            (emit-label regs-defaulted)))
        (when (< register-arg-count
-                (sb!kernel:values-type-max-value-count type))
+                (values-type-max-value-count type))
          (inst mov esp-tn ebx-tn)))
       ((<= nvals 7)
        ;; The number of bytes depends on the relative jump instructions.
@@ -437,7 +434,6 @@
            (let ((defaults (defaults)))
              (when defaults
                (assemble (*elsewhere*)
-                 (trace-table-entry trace-table-fun-prologue)
                  (emit-label default-stack-slots)
                  (dolist (default defaults)
                    (emit-label (car default))
@@ -449,8 +445,7 @@
                      ;; above.
                      (inst push edx-tn))
                    (inst mov (second default) eax-tn))
-                 (inst jmp defaulting-done)
-                 (trace-table-entry trace-table-normal)))))))
+                 (inst jmp defaulting-done)))))))
       (t
        ;; 91 bytes for this branch.
        (let ((regs-defaulted (gen-label))
@@ -558,7 +553,7 @@
         (variable-values (gen-label))
         (stack-values (gen-label))
         (done (gen-label)))
-    (when (sb!kernel:values-type-may-be-single-value-p type)
+    (when (values-type-may-be-single-value-p type)
       (inst jmp :c variable-values)
       (cond ((location= start (first *register-arg-tns*))
              (inst push (first *register-arg-tns*))
@@ -570,7 +565,7 @@
       (emit-label variable-values))
     ;; The stack frame is burnt and RETurned from if there are no
     ;; stack values. In this case quickly reallocate sufficient space.
-    (when (<= (sb!kernel:values-type-min-value-count type)
+    (when (<= (values-type-min-value-count type)
               register-arg-count)
       (inst cmp nargs (fixnumize register-arg-count))
       (inst jmp :g stack-values)
@@ -582,7 +577,7 @@
     (loop
       for arg in *register-arg-tns*
       for i downfrom -1
-      for j below (sb!kernel:values-type-max-value-count type)
+      for j below (values-type-max-value-count type)
       do (storew arg args i))
     (move start args)
     (move count nargs)
@@ -664,12 +659,10 @@
   (:ignore nfp arg-locs args callee)
   (:node-var node)
   (:generator 5
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
-    (default-unknown-values vop values nvals node)
-    (trace-table-entry trace-table-normal)))
+    (default-unknown-values vop values nvals node)))
 
 ;;; Non-TR local call for a variable number of return values passed according
 ;;; to the unknown values convention. The results are the start of the values
@@ -685,13 +678,11 @@
   (:vop-var vop)
   (:node-var node)
   (:generator 20
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
     (note-this-location vop :unknown-return)
-    (receive-unknown-values values-start nvals start count node)
-    (trace-table-entry trace-table-normal)))
+    (receive-unknown-values values-start nvals start count node)))
 
 ;;;; local call with known values return
 
@@ -712,12 +703,10 @@
   (:ignore args res save nfp callee)
   (:vop-var vop)
   (:generator 5
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
-    (note-this-location vop :known-return)
-    (trace-table-entry trace-table-normal)))
+    (note-this-location vop :known-return)))
 
 ;;; From Douglas Crosher
 ;;; Return from known values call. We receive the return locations as
@@ -733,12 +722,10 @@
   (:vop-var vop)
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Zot all of the stack except for the old-fp and return-pc.
     (inst mov esp-tn ebp-tn)
     (inst pop ebp-tn)
-    (inst ret)
-    (trace-table-entry trace-table-normal)))
+    (inst ret)))
 
 ;;;; full call
 ;;;
@@ -857,7 +844,6 @@
                                (if (eq return :tail) 0 10)
                                15
                                (if (eq return :unknown) 25 0))
-               (trace-table-entry trace-table-call-site)
 
                ;; This has to be done before the frame pointer is
                ;; changed! EAX stores the 'lexical environment' needed
@@ -971,8 +957,7 @@
                     '((note-this-location vop :unknown-return)
                       (receive-unknown-values values-start nvals start count
                                               node)))
-                   (:tail))
-               (trace-table-entry trace-table-normal)))))
+                   (:tail))))))
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
@@ -1020,7 +1005,6 @@
   (:ignore value)
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Drop stack above old-fp
     (inst mov esp-tn ebp-tn)
     ;; Clear the multiple-value return flag
@@ -1062,7 +1046,6 @@
     (when (= nvals 1)
       ;; This is handled in RETURN-SINGLE.
       (error "nvalues is 1"))
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Establish the values pointer and values count.
     (inst lea ebx (make-ea :dword :base ebp-tn
                            :disp (* sp->fp-offset n-word-bytes)))
@@ -1100,9 +1083,7 @@
                                :disp (frame-byte-offset
                                       (+ sp->fp-offset
                                          (tn-offset return-pc)))))
-           (inst ret)))
-
-    (trace-table-entry trace-table-normal)))
+           (inst ret)))))
 
 ;;; Do unknown-values return of an arbitrary number of values (passed
 ;;; on the stack.) We check for the common case of a single return
@@ -1125,7 +1106,6 @@
   (:node-var node)
   (:generator 13
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     (unless (policy node (> space speed))
       ;; Check for the single case.
       (let ((not-single (gen-label)))
@@ -1144,8 +1124,7 @@
         (emit-label not-single)))
     (move esi vals)
     (move ecx nvals)
-    (inst jmp (make-fixup 'return-multiple :assembly-routine))
-    (trace-table-entry trace-table-normal)))
+    (inst jmp (make-fixup 'return-multiple :assembly-routine))))
 
 ;;;; XEP hackery
 
@@ -1476,7 +1455,7 @@
                 (:generator 1000
                   (error-call vop ',error ,@args)))))
   (def arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs)
+    sb!c::%arg-count-error nargs fname)
   (def type-check-error object-not-type-error sb!c::%type-check-error
     object type)
   (def layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
