@@ -477,7 +477,7 @@
        ;; In the vast majority of cases location corresponds to the position
        ;; in list. The only exceptions are when there are non-local slots
        ;; before the one we want.
-       (let* ((slots (wrapper-slots (wrapper-of instance)))
+       (let* ((slots (layout-slot-list (layout-of instance)))
               (guess (nth position slots)))
          (if (eql position (slot-definition-location guess))
              (slot-definition-name guess)
@@ -506,9 +506,11 @@
   (declare (ignore initargs))
   (allocate-condition (class-name class)))
 
-(defmethod allocate-instance ((class built-in-class) &rest initargs)
-  (declare (ignore initargs))
-  (error "Cannot allocate an instance of ~S." class)) ; So sayeth AMOP
+(macrolet ((def (name class)
+             `(defmethod ,name ((class ,class) &rest initargs)
+                (declare (ignore initargs))
+                (error "Cannot allocate an instance of ~S." class))))
+  (def allocate-instance system-class))
 
 ;;; AMOP says that CLASS-SLOTS signals an error for unfinalized classes.
 (defmethod class-slots :before ((class slot-class))
