@@ -61,9 +61,10 @@ os_vm_size_t os_vm_page_size;
 #include <dirent.h>   /* For the opendir()/readdir() wrappers */
 #include <sys/socket.h> /* For the socket() wrapper */
 static void netbsd_init();
+static os_vm_size_t max_allocation_size;
 #endif /* __NetBSD__ */
 
-#ifdef __FreeBSD__
+#if defined LISP_FEATURE_FREEBSD
 #include <sys/sysctl.h>
 #if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
 #include <sys/umtx.h>
@@ -72,6 +73,12 @@ static void netbsd_init();
 static void freebsd_init();
 #endif /* __FreeBSD__ */
 
+#ifdef __DragonFly__
+#include <sys/sysctl.h>
+
+static void dragonfly_init();
+#endif /* __DragonFly__ */
+
 #ifdef __OpenBSD__
 #include <sys/types.h>
 #include <sys/resource.h>
@@ -92,12 +99,14 @@ os_init(char *argv[], char *envp[])
 
 #ifdef __NetBSD__
     netbsd_init();
-#elif defined(__FreeBSD__)
+#elif defined(LISP_FEATURE_FREEBSD)
     freebsd_init();
 #elif defined(__OpenBSD__)
     openbsd_init();
 #elif defined(LISP_FEATURE_DARWIN)
     darwin_init();
+#elif defined(__DragonFly__)
+    dragonfly_init();
 #endif
 }
 
@@ -107,7 +116,8 @@ os_context_sigmask_addr(os_context_t *context)
     /* (Unlike most of the other context fields that we access, the
      * signal mask field is a field of the basic, outermost context
      * struct itself both in FreeBSD 4.0 and in OpenBSD 2.6.) */
-#if defined(__FreeBSD__)  || defined(__NetBSD__) || defined(LISP_FEATURE_DARWIN)
+#if defined(LISP_FEATURE_FREEBSD) || defined(__NetBSD__) || defined(LISP_FEATURE_DARWIN) \
+    || defined(__DragonFly__)
     return &context->uc_sigmask;
 #elif defined (__OpenBSD__)
     return &context->sc_mask;
@@ -124,7 +134,36 @@ os_validate(os_vm_address_t addr, os_vm_size_t len)
     if (addr)
         flags |= MAP_FIXED;
 
-    addr = mmap(addr, len, OS_VM_PROT_ALL, flags, -1, 0);
+#ifdef __NetBSD__
+    if (addr) {
+        os_vm_address_t curaddr = addr;
+
+        while (len > 0) {
+            os_vm_address_t resaddr;
+            os_vm_size_t curlen = MIN(max_allocation_size, len);
+
+            resaddr = mmap(curaddr, curlen, OS_VM_PROT_ALL, flags, -1, 0);
+
+            if (resaddr == (os_vm_address_t) - 1) {
+                perror("mmap");
+
+                while (curaddr > addr) {
+                    curaddr -= max_allocation_size;
+                    munmap(curaddr, max_allocation_size);
+                }
+
+                return NULL;
+            }
+
+            curaddr += curlen;
+            len -= curlen;
+        }
+    } else {
+#endif
+     addr = mmap(addr, len, OS_VM_PROT_ALL, flags, -1, 0);
+#ifdef __NetBSD__
+    }
+#endif
 
     if (addr == MAP_FAILED) {
         perror("mmap");
@@ -244,7 +283,7 @@ os_install_interrupt_handlers(void)
                                                  mach_error_memory_fault_handler);
 #else
     undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
-#ifdef LISP_FEATURE_FREEBSD
+#if defined(LISP_FEATURE_FREEBSD) && !defined(__GLIBC__)
                                                  (__siginfohandler_t *)
 #endif
                                                  memory_fault_handler);
@@ -312,18 +351,32 @@ static void netbsd_init()
     /* NetBSD counts mmap()ed space against the process's data size limit,
      * so yank it up. This might be a nasty thing to do? */
     getrlimit (RLIMIT_DATA, &rl);
-    /* Amazingly for such a new port, the provenance and meaning of
-       this number are unknown.  It might just mean REALLY_BIG_LIMIT,
-       or possibly it should be calculated from dynamic space size.
-       -- CSR, 2004-04-08 */
-    rl.rlim_cur = 1073741824;
-    if (setrlimit (RLIMIT_DATA, &rl) < 0) {
-        fprintf (stderr,
-                 "RUNTIME WARNING: unable to raise process data size limit:\n\
-  %s.\n\
+    if (rl.rlim_cur < rl.rlim_max) {
+        rl.rlim_cur = rl.rlim_max;
+        if (setrlimit (RLIMIT_DATA, &rl) < 0) {
+            fprintf (stderr,
+                     "RUNTIME WARNING: unable to raise process data size limit:\n\
+%s.\n\
 The system may fail to start.\n",
-                 strerror(errno));
+                     strerror(errno));
+        }
     }
+    max_allocation_size = (os_vm_size_t)((rl.rlim_cur / 2) &
+      ~(32 * 1024 * 1024));
+
+#ifdef LISP_FEATURE_X86
+    {
+        size_t len;
+        int sse;
+
+        len = sizeof(sse);
+        if (sysctlbyname("machdep.sse", &sse, &len,
+                         NULL, 0) == 0 && sse != 0) {
+            /* Use the SSE detector */
+            fast_bzero_pointer = fast_bzero_detect;
+        }
+     }
+#endif /* LISP_FEATURE_X86 */
 }
 
 /* Various routines in NetBSD's C library are compatibility wrappers
@@ -372,8 +425,10 @@ _socket(int domain, int type, int protocol)
 }
 #endif /* __NetBSD__ */
 
-#ifdef __FreeBSD__
+#if defined(LISP_FEATURE_FREEBSD)
+#ifndef __GLIBC__
 extern int getosreldate(void);
+#endif
 
 int sig_memory_fault;
 
@@ -381,10 +436,14 @@ static void freebsd_init()
 {
     /* Memory fault signal on FreeBSD was changed from SIGBUS to
      * SIGSEGV. */
+#ifdef __GLIBC__
+        sig_memory_fault = SIGSEGV;
+#else
     if (getosreldate() < 700004)
         sig_memory_fault = SIGBUS;
     else
         sig_memory_fault = SIGSEGV;
+#endif
 
     /* Quote from sbcl-devel (NIIMI Satoshi): "Some OSes, like FreeBSD
      * 4.x with GENERIC kernel, does not enable SSE support even on
@@ -444,6 +503,58 @@ futex_wake(int *lock_word, int n)
 #endif
 #endif /* __FreeBSD__ */
 
+#ifdef __DragonFly__
+static void dragonfly_init()
+{
+#ifdef LISP_FEATURE_X86
+    size_t len;
+    int instruction_sse;
+
+    len = sizeof(instruction_sse);
+    if (sysctlbyname("hw.instruction_sse", &instruction_sse, &len,
+                     NULL, 0) == 0 && instruction_sse != 0) {
+        /* Use the SSE detector */
+        fast_bzero_pointer = fast_bzero_detect;
+    }
+#endif /* LISP_FEATURE_X86 */
+}
+
+
+#if defined(LISP_FEATURE_SB_THREAD) && defined(LISP_FEATURE_SB_FUTEX) \
+    && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+int
+futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
+{
+    int ret;
+
+    if (sec < 0)
+        ret = umtx_sleep(lock_word, oldval, 0);
+    else {
+        int count = usec + 1000000 * sec;
+        ret = umtx_sleep(lock_word, oldval, count);
+    }
+
+    if (ret == 0) return 0;
+    else {
+        switch (errno) {
+        case EWOULDBLOCK: // Operation timed out
+            return 1;
+        case EINTR:
+            return 2;
+        default: // Such as EINVAL or EBUSY
+            return -1;
+        }
+    }
+}
+
+int
+futex_wake(int *lock_word, int n)
+{
+    return umtx_wakeup(lock_word, n);
+}
+#endif
+#endif /* __DragonFly__ */
+
 #ifdef LISP_FEATURE_DARWIN
 /* defined in ppc-darwin-os.c instead */
 #elif defined(LISP_FEATURE_FREEBSD)
@@ -456,7 +567,9 @@ os_get_runtime_executable_path(int external)
 {
     char path[PATH_MAX + 1];
 
+#ifndef __GLIBC__
     if (getosreldate() >= 600024) {
+#endif
         /* KERN_PROC_PATHNAME is available */
         size_t len = PATH_MAX + 1;
         int mib[4];
@@ -467,6 +580,7 @@ os_get_runtime_executable_path(int external)
         mib[3] = -1;
         if (sysctl(mib, 4, &path, &len, NULL, 0) != 0)
             return NULL;
+#ifndef __GLIBC__
     } else {
         int size;
         size = readlink("/proc/curproc/file", path, sizeof(path) - 1);
@@ -474,6 +588,21 @@ os_get_runtime_executable_path(int external)
             return NULL;
         path[size] = '\0';
     }
+#endif
+    if (strcmp(path, "unknown") == 0)
+        return NULL;
+    return copied_string(path);
+}
+#elif defined(LISP_FEATURE_DRAGONFLY)
+char *
+os_get_runtime_executable_path(int external)
+{
+    char path[PATH_MAX + 1];
+    int size = readlink("/proc/curproc/file", path, sizeof(path) - 1);
+    if (size < 0)
+        return NULL;
+    path[size] = '\0';
+
     if (strcmp(path, "unknown") == 0)
         return NULL;
     return copied_string(path);
@@ -487,7 +616,7 @@ os_get_runtime_executable_path(int external)
         return copied_string("/proc/curproc/file");
     return NULL;
 }
-#else /* Not DARWIN or FREEBSD or NETBSD or OPENBSD */
+#else /* Not DARWIN or FREEBSD or NETBSD or OPENBSD or DragonFly */
 char *
 os_get_runtime_executable_path(int external)
 {
