@@ -126,7 +126,7 @@ scavenge(lispobj *start, sword_t n_words)
         lispobj object = *object_ptr;
 #ifdef LISP_FEATURE_GENCGC
         if (forwarding_pointer_p(object_ptr))
-            lose("unexpect forwarding pointer in scavenge: %p, start=%p, n=%l\n",
+            lose("unexpect forwarding pointer in scavenge: %p, start=%p, n=%ld\n",
                  object_ptr, start, n_words);
 #endif
         if (is_lisp_pointer(object)) {
@@ -250,7 +250,7 @@ trans_code(struct code *code)
     /* prepare to transport the code vector */
     l_code = (lispobj) LOW_WORD(code) | OTHER_POINTER_LOWTAG;
 
-    ncode_words = fixnum_value(code->code_size);
+    ncode_words = fixnum_word_value(code->code_size);
     nheader_words = HeaderValue(code->header);
     nwords = ncode_words + nheader_words;
     nwords = CEILING(nwords, 2);
@@ -333,7 +333,7 @@ scav_code_header(lispobj *where, lispobj object)
     struct simple_fun *function_ptr; /* untagged pointer to entry point */
 
     code = (struct code *) where;
-    n_code_words = fixnum_value(code->code_size);
+    n_code_words = fixnum_word_value(code->code_size);
     n_header_words = HeaderValue(object);
     n_words = n_code_words + n_header_words;
     n_words = CEILING(n_words, 2);
@@ -381,7 +381,7 @@ size_code_header(lispobj *where)
 
     code = (struct code *) where;
 
-    ncode_words = fixnum_value(code->code_size);
+    ncode_words = fixnum_word_value(code->code_size);
     nheader_words = HeaderValue(code->header);
     nwords = ncode_words + nheader_words;
     nwords = CEILING(nwords, 2);
@@ -1883,7 +1883,7 @@ scan_weak_hash_tables (void)
 static sword_t
 scav_lose(lispobj *where, lispobj object)
 {
-    lose("no scavenge function for object 0x%08x (widetag 0x%x)\n",
+    lose("no scavenge function for object %p (widetag 0x%x)\n",
          (uword_t)object,
          widetag_of(*where));
 
