@@ -62,7 +62,7 @@
 
 (defpackage "TEST-ORIGINAL" (:nicknames "A-NICKNAME"))
 
-(assert (raises-error? (defpackage "A-NICKNAME")))
+(assert-error (defpackage "A-NICKNAME"))
 
 (assert (eql (find-package "A-NICKNAME")
              (find-package "TEST-ORIGINAL")))
@@ -289,6 +289,13 @@ if a restart was invoked."
                   (assert (equal (list :foo) (simple-condition-format-arguments c)))
                   :good)))))
 
+;; X3J13 writeup HASH-TABLE-PACKAGE-GENERATORS says
+;;  "An argument of NIL is treated as an empty list of packages."
+;; This used to fail with "NIL does not name a package"
+(with-test (:name :with-package-iterator-nil-list)
+  (with-package-iterator (iter '() :internal)
+    (print (nth-value 1 (iter)))))
+
 ;;; MAKE-PACKAGE error in another thread blocking FIND-PACKAGE & FIND-SYMBOL
 (with-test (:name :bug-511072 :skipped-on '(not :sb-thread))
   (let* ((p (make-package :bug-511072))
@@ -651,3 +658,108 @@ if a restart was invoked."
              (delete-package (gensym)))))
     (assert ok)
     (assert (not result))))
+
+;; WITH-PACKAGE-ITERATOR isn't well-exercised by tests (though LOOP uses it)
+;; so here's a basic correctness test with some complications involving
+;; shadowing symbols.
+(make-package "P1" :use '("SB-FORMAT"))
+(make-package "P2")
+(export 'p1::foo 'p1)
+(shadow "FORMAT-ERROR" 'p1)
+(make-package "A" :use '("SB-FORMAT" "P1" "P2"))
+(shadow '("PROG2" "FOO") 'a)
+(intern "BLAH" "P2")
+(export 'p2::(foo bar baz) 'p2)
+(export 'a::goodfun 'a)
+
+(with-test (:name :with-package-iterator)
+  (let ((tests '((:internal) (:external) (:inherited)
+                 (:internal :inherited)
+                 (:internal :external)
+                 (:external :inherited)
+                 (:internal :external :inherited)))
+        (maximum-answer
+         '(;; symbols visible in A
+           (a::prog2 :internal "A")
+           (a::foo :internal "A")
+           (a:goodfun :external "A")
+           (p2:bar :inherited "A")
+           (p2:baz :inherited "A")
+           (sb-format:%compiler-walk-format-string :inherited "A")
+           (sb-format:format-error :inherited "A")
+           ;; ... P1
+           (p1:foo :external "P1")
+           (p1::format-error :internal "P1")
+           (sb-format:%compiler-walk-format-string :inherited "P1")
+           ;; ... P2
+           (p2::blah :internal "P2")
+           (p2:foo :external "P2")
+           (p2:bar :external "P2")
+           (p2:baz :external "P2"))))
+    ;; Compile a new function to test each combination of
+    ;; accessibility-kind since the macro doesn't eval them.
+    (dolist (access tests)
+      ; (print `(testing ,access))
+      (let ((f (compile
+                nil
+                `(lambda ()
+                   (with-package-iterator (iter '(p1 a p2) ,@access)
+                     (let (res)
+                       (loop
+                        (multiple-value-bind (foundp sym access pkg) (iter)
+                          (if foundp
+                              (push (list sym access (package-name pkg)) res)
+                              (return))))
+                       res))))))
+        (let ((answer (funcall f))
+              (expect (remove-if-not (lambda (x) (member (second x) access))
+                                     maximum-answer)))
+          ;; exactly as many results as expected
+          (assert (equal (length answer) (length expect)))
+          ;; each result is right
+          (assert (equal (length (intersection answer expect :test #'equal))
+                         (length expect))))))))
+
+;; Assert that changes in size of a package-hashtable's symbol vector
+;; do not cause WITH-PACKAGE-ITERATOR to crash. The vector shouldn't grow,
+;; because it is not permitted to INTERN new symbols, but it can shrink
+;; because it is expressly permitted to UNINTERN the current symbol.
+;; (In fact we allow INTERN, but that's beside the point)
+(with-test (:name :with-package-iterator-and-mutation)
+  (flet ((table-size (pkg)
+           (length (sb-impl::package-hashtable-table
+                    (sb-impl::package-internal-symbols pkg)))))
+    (let* ((p (make-package (string (gensym))))
+           (initial-table-size (table-size p))
+           (strings
+            '("a" "b" "c" "d" "e" "f" "g" "h" "i" "j" "k" "l" "m")))
+      (dolist (x strings)
+        (intern x p))
+      (let ((grown-table-size (table-size p)))
+        (assert (> grown-table-size initial-table-size))
+        (let ((n 0))
+          (with-package-iterator (iter p :internal)
+            (loop (multiple-value-bind (foundp sym) (iter)
+                    (cond (foundp
+                           (incf n)
+                           (unintern sym p))
+                          (t
+                           (return)))))
+            (assert (= n (length strings)))
+            ;; while we're at it, assert that calling the iterator
+            ;; a couple more times returns nothing.
+            (dotimes (i 2)
+              (assert (not (iter))))))
+        (let ((shrunk-table-size (table-size p)))
+          (assert (< shrunk-table-size grown-table-size)))))))
+
+(with-test (:name :export-inaccessible-lookalike)
+  (make-package "E1")
+  (make-package "E2")
+  (export (intern "A" "E2") 'e2)
+  (multiple-value-bind (answer condition)
+      (ignore-errors  (export (intern "A" "E1") 'e2))
+    (assert (and (not answer)
+                 (and (typep condition 'sb-kernel:simple-package-error)
+                      (search "not accessible"
+                              (simple-condition-format-control condition)))))))
