@@ -13,6 +13,8 @@
 
 (in-package "CL-USER")
 
+(load "compiler-test-util.lisp")
+
 (assert (equal (symbol-name '#:|fd\sA|) "fdsA"))
 
 ;;; Prior to sbcl-0.7.2.10, SBCL disobeyed the ANSI requirements on
@@ -277,6 +279,8 @@
 (with-test (:name :read-in-package-syntax)
   (assert (equal '(sb-c::a (sb-kernel::x sb-kernel::y) sb-c::b)
                  (read-from-string "sb-c::(a sb-kernel::(x y) b)")))
+  (assert (equal '(cl-user::yes-this-is-sbcl)
+                 (read-from-string "cl-user::(#+sbcl yes-this-is-sbcl)")))
   #+sb-package-locks
   (assert (eq :violated!
               (handler-case
@@ -286,8 +290,44 @@
 
 (with-test (:name :bug-309070)
   (with-timeout 10
-    (assert (raises-error? (read-from-string "10e10000000000000000000")
-                           sb-kernel:reader-impossible-number-error))))
+    (assert-error (read-from-string "10e10000000000000000000")
+                  sb-kernel:reader-impossible-number-error)))
 
 (with-test (:name :bug-1095918)
   (assert (= (length `#3(1)) 3)))
+
+(with-test (:name :obscure-reader-package-usage)
+  ;; commit 8fd604 cause a bug in reading "::(foo bar)" which tried
+  ;; to treat the package-designator as a string, but in this case
+  ;; it is hardcoded to *keyword-package*.
+  (assert (equal (read-from-string "::(foo bar)") '(:foo :bar))))
+
+#+x86-64
+;; I do not know the complete list of platforms for which this test
+;; will not cons, but there were four different heap allocations
+;; instead of using dx allocation or a recyclable resource:
+;;  - most obviously, a 128-character buffer per invocation of READ
+;;  - calling SUBSEQ for package names
+;;  - multiple-value-call in WITH-CHAR-MACRO-RESULT
+;;  - the initial cons cell in READ-LIST
+(with-test (:name :read-does-not-cons-per-se)
+  (flet ((test-reading (string)
+           (let ((s (make-string-input-stream string)))
+             (read s) ; once outside the loop, to make A-SYMBOL
+             (ctu:assert-no-consing
+              (progn (file-position s 0)
+                     (read s))
+              40000))))
+    ;; These each used to produce at least 20 MB of garbage,
+    ;; a result of using 128-character (= 512 bytes for Unicode) buffers.
+    ;; Now we use exactly one buffer, or maybe two for package + symbol-name.
+    (test-reading "4.0s0")
+    (test-reading "COMMON-LISP-USER::A-SYMBOL")
+    (test-reading "()")
+    (test-reading "#\\-") ; should not copy the token buffer
+    ))
+
+(with-test (:name :sharp-star-empty-multiple-escapes)
+  (assert (eq (handler-case (read-from-string "#*101||1")
+                (sb-int:simple-reader-error () :win))
+              :win)))
