@@ -197,7 +197,7 @@
 ;;; bug 46c
 (with-test (:name :coerce-function-on-macro)
   (dolist (fun '(and if))
-    (assert (raises-error? (coerce fun 'function)))))
+    (assert-error (coerce fun 'function))))
 
 (dotimes (i 100)
   (let ((x (make-array 0 :element-type `(unsigned-byte ,(1+ i)))))
@@ -470,3 +470,93 @@
   (assert (sb-c::type= (sb-c::single-value-type
                         (sb-c::values-specifier-type '(values &optional character)))
                        (sb-c::specifier-type '(or null character)))))
+
+;; lp#1317308 - TYPE-OF must not return a type specifier
+;; involving AND,EQL,MEMBER,NOT,OR,SATISFIES,or VALUES.
+(with-test (:name :ANSIly-report-hairy-array-type)
+  (let ((simp-t (make-array 9))
+        (simp-bit (make-array 16 :element-type 'bit)))
+    ;; TYPE-OF doesn't have an optimization that returns a constant specifier
+    ;; from a non-constant array of known type. If it did, we'd probably
+    ;; want to check that these results are all equal:
+    ;;  - the runtime-determined type
+    ;;  - the compile-time-determined constant type
+    ;;  - the compile-time-determined type of an equivalent object
+    ;;    that is in fact a compile-time constant
+    (flet ((our-type-of (x) (sb-kernel:type-specifier (sb-kernel:ctype-of x))))
+      (let ((hairy-t (make-array 3 :displaced-to simp-t)))
+        (assert (equal (our-type-of hairy-t)
+                       '(and (vector t 3) (not simple-array))))
+        (assert (equal (type-of hairy-t) '(vector t 3))))
+      (let ((hairy-t (make-array '(3 2) :displaced-to simp-t)))
+        (assert (equal (our-type-of hairy-t)
+                       '(and (array t (3 2)) (not simple-array))))
+        (assert (equal (type-of hairy-t) '(array t (3 2)))))
+      (let ((hairy-bit
+             (make-array 5 :displaced-to simp-bit :element-type 'bit)))
+        (assert (equal (our-type-of hairy-bit)
+                       '(and (bit-vector 5) (not simple-array))))
+        (assert (equal (type-of hairy-bit) '(bit-vector 5)))))))
+
+(with-test (:name :bug-309098)
+  (let ((u `(or ,@(map 'list (lambda (x) `(array ,(sb-vm:saetp-specifier x)))
+                       sb-vm:*specialized-array-element-type-properties*))))
+    (assert (equal (multiple-value-list (subtypep 'array u)) '(t t)))))
+
+(with-test (:name :bug-1258716)
+  (let ((intersection (sb-kernel:type-intersection
+                       (sb-kernel:specifier-type 'simple-vector)
+                       (sb-kernel:specifier-type `(vector #:unknown)))))
+    (assert (sb-kernel:array-type-p intersection))
+    ;; and not *wild-type*
+    (assert (sb-kernel:type= (sb-kernel:array-type-specialized-element-type intersection)
+                             sb-kernel:*universal-type*))))
+
+(with-test (:name :unparse-safely)
+  (let* ((intersection (sb-kernel:type-intersection
+                        (sb-kernel:specifier-type '(vector (or bit character)))
+                        (sb-kernel:specifier-type `(vector (or bit symbol)))))
+         (round-trip (sb-kernel:specifier-type
+                     (sb-kernel:type-specifier intersection))))
+    (assert (sb-kernel:type= intersection round-trip))
+    (assert (sb-kernel:array-type-p intersection))
+    ;; and not *wild-type*
+    (assert (sb-kernel:type/= (sb-kernel:array-type-specialized-element-type intersection)
+                              (sb-kernel:specifier-type 'bit)))))
+
+(in-package "SB-KERNEL")
+(test-util:with-test (:name :partition-array-into-simple/hairy)
+  ;; Some tests that (simple-array | hairy-array) = array
+  ;; At present this works only for wild element-type.
+  (multiple-value-bind (eq winp)
+      (type= (specifier-type '(not (and array (not simple-array))))
+             (specifier-type '(or (not array) simple-array)))
+    (assert (and eq winp)))
+
+  ;; if X is neither simple-array nor hairy-array, it is not an array
+  (assert (type= (specifier-type '(and (not simple-array)
+                                       (not (and array (not simple-array)))))
+                 (specifier-type '(not array))))
+
+  ;; (simple-array * (*)) = (AND (NOT <hairy-array>) VECTOR) etc
+  (flet ((try (unrestricted simple)
+           (assert (type= (specifier-type simple)
+                          (type-intersection
+                           (specifier-type
+                            '(not (and array (not simple-array))))
+                           (specifier-type unrestricted))))))
+    (try 'vector '(simple-array * (*)))
+    (try '(vector t) 'simple-vector)
+    (try 'bit-vector 'simple-bit-vector)
+    (try 'string 'simple-string)
+    #+sb-unicode(try 'character-string 'simple-character-string)
+    (try 'base-string 'simple-base-string))
+
+  ;; if X is a known string and not an array-header
+  ;; it must be a SIMPLE-STRING
+  (assert (type= (type-intersection
+                  (specifier-type 'string)
+                  (specifier-type
+                   '(not (or (and simple-array (not vector))
+                             (and array (not simple-array))))))
+                 (specifier-type 'simple-string))))
